@@ -33,7 +33,11 @@ typedef int32_t INT32;
 typedef uint8_t UINT8;
 typedef uint16_t UINT16;
 typedef uint32_t UINT32;
-#define NORM_API_LINKAGE 
+#if (defined __GNUC__ && __GNUC__ >= 4)
+#define NORM_API_LINKAGE __attribute__ ((visibility ("default")))
+#else
+#define NORM_API_LINKAGE
+#endif
 #endif // if/else WIN32/UNIX
 
 ////////////////////////////////////////////////////////////
@@ -59,8 +63,8 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define NORM_VERSION_MAJOR 1
-#define NORM_VERSION_MINOR 0
-#define NORM_VERSION_PATCH 0
+#define NORM_VERSION_MINOR 5
+#define NORM_VERSION_PATCH 7
 
 /** NORM API Types */
 typedef const void* NormInstanceHandle;
@@ -187,7 +191,7 @@ typedef enum NormEventType
     NORM_GRTT_UPDATED,
     NORM_CC_ACTIVE,
     NORM_CC_INACTIVE,
-    NORM_ACKING_NODE_NEW,        // whe NormSetAutoAcking xxx
+    NORM_ACKING_NODE_NEW,        // whe NormSetAutoAcking 
     NORM_SEND_ERROR,             // ICMP error (e.g. destination unreachable)
     NORM_USER_TIMEOUT            // issues when timeout set by NormSetUserTimer() expires
 } NormEventType;
@@ -201,6 +205,11 @@ typedef struct
 } NormEvent;
     
 
+// For setting custom NORM_OBJECT_DATA alloc/free functions
+typedef char* (*NormAllocFunctionHandle)(size_t);
+typedef void (*NormFreeFunctionHandle)(char*);
+
+
 /** NORM API General Initialization and Operation Functions */
 
 NORM_API_LINKAGE 
@@ -208,6 +217,9 @@ int NormGetVersion(int* major DEFAULT((int*)0),
                    int* minor DEFAULT((int*)0), 
                    int* patch  DEFAULT((int*)0));
 
+
+
+
 NORM_API_LINKAGE 
 NormInstanceHandle NormCreateInstance(bool priorityBoost DEFAULT(false));
 
@@ -256,6 +268,11 @@ const NormDescriptor NORM_DESCRIPTOR_INVALID;
 NORM_API_LINKAGE 
 NormDescriptor NormGetDescriptor(NormInstanceHandle instanceHandle);
 
+NORM_API_LINKAGE 
+void NormSetAllocationFunctions(NormInstanceHandle      instance, 
+                                NormAllocFunctionHandle allocFunc, 
+                                NormFreeFunctionHandle  freeFunc);
+
 /** NORM Session Creation and Control Functions */
 
 NORM_API_LINKAGE 
@@ -288,6 +305,11 @@ void NormCancelUserTimer(NormSessionHandle sessionHandle);
 NORM_API_LINKAGE 
 NormNodeId NormGetLocalNodeId(NormSessionHandle sessionHandle);
 
+NORM_API_LINKAGE
+bool NormGetAddress(NormSessionHandle   sessionHandle,
+                    char*               addrBuffer, 
+                    unsigned int*       bufferLen,
+                    UINT16*             port DEFAULT((UINT16*)0));
 
 NORM_API_LINKAGE
 UINT16 NormGetRxPort(NormSessionHandle sessionHandle);
@@ -306,6 +328,8 @@ void NormSetTxOnly(NormSessionHandle sessionHandle,
                    bool              txOnly,
                    bool              connectToSessionAddress DEFAULT(false));
 
+NORM_API_LINKAGE
+void NormSetId(NormSessionHandle sessionHandle, NormNodeId normId);
 
 // This does not affect the rx_socket binding if already bound (sender or receiver already started)
 // (i.e., just affects where NORM packets are sent)
@@ -315,6 +339,13 @@ bool NormChangeDestination(NormSessionHandle sessionHandle,
                            UINT16            sessionPort,
                            bool              connectToSessionAddress DEFAULT(false));
 
+NORM_API_LINKAGE
+void NormSetServerListener(NormSessionHandle sessionHandle, bool state);
+
+NORM_API_LINKAGE
+bool NormTransferSender(NormSessionHandle sessionHandle, NormNodeHandle sender);
+
+
 NORM_API_LINKAGE 
 void NormSetRxPortReuse(NormSessionHandle sessionHandle,
                         bool              enableReuse,
@@ -410,7 +441,8 @@ bool NormStartSender(NormSessionHandle  sessionHandle,
                      UINT32             bufferSpace,
                      UINT16             segmentSize,
                      UINT16             numData,
-                     UINT16             numParity);
+                     UINT16             numParity,
+                     UINT8              fecId DEFAULT(0));
 
 NORM_API_LINKAGE 
 void NormStopSender(NormSessionHandle sessionHandle);
@@ -583,6 +615,9 @@ bool NormSendCommand(NormSessionHandle  sessionHandle,
 
 NORM_API_LINKAGE 
 void NormCancelCommand(NormSessionHandle  sessionHandle);
+
+NORM_API_LINKAGE 
+void NormSetSynStatus(NormSessionHandle sessionHandle, bool state);
         
 /* NORM Receiver Functions */
 
@@ -646,6 +681,14 @@ NORM_API_LINKAGE
 void NormNodeSetRxRobustFactor(NormNodeHandle   remoteSender,
                                int              robustFactor);
 
+NORM_API_LINKAGE
+bool NormPreallocateRemoteSender(NormSessionHandle  sessionHandle,
+                                 unsigned long      bufferSize,
+                                 UINT16             segmentSize, 
+                                 UINT16             numData, 
+                                 UINT16             numParity,
+                                 unsigned int       streamBufferSize DEFAULT(0));
+
 NORM_API_LINKAGE 
 bool NormStreamRead(NormObjectHandle   streamHandle,
                     char*              buffer,
@@ -706,6 +749,12 @@ const char* NormDataAccessData(NormObjectHandle objectHandle);
 NORM_API_LINKAGE 
 char* NormDataDetachData(NormObjectHandle objectHandle);
 
+NORM_API_LINKAGE 
+char* NormAlloc(size_t numBytes); 
+
+NORM_API_LINKAGE 
+void NormFree(char* dataPtr);
+
 NORM_API_LINKAGE 
 NormNodeHandle NormObjectGetSender(NormObjectHandle objectHandle);
 
