@@ -7,7 +7,15 @@
 
 #include <stdio.h>
 
+#define USE_PROTO_TREE 1  // for more better performing NormObjectTable?
+
+#ifdef USE_PROTO_TREE
+#include "protoTree.h"
+
+class NormObject : public ProtoSortedTree::Item
+#else
 class NormObject
+#endif // if/else USE_PROTO_TREE
 {
     friend class NormObjectTable;
     
@@ -44,16 +52,43 @@ class NormObject
         
         // This must be reset after each update
         void SetNotifyOnUpdate(bool state)
-        {
-            notify_on_update = state;
-        }
+            {notify_on_update = state;}
         
         // Object information
         NormObject::Type GetType() const {return type;}
         const NormObjectId& GetId() const {return transport_id;}  
         const NormObjectSize& GetSize() const {return object_size;}
+        
+        void SetId(const NormObjectId transportId)
+            {transport_id = transportId;}
+        
+        
+        UINT16 GetSegmentSize() const {return segment_size;}
+        UINT8 GetFecId() const {return fec_id;}
+        UINT16 GetFecMaxBlockLen() const {return ndata;}
+        UINT16 GetFecNumParity() const {return nparity;}
+        UINT8 GetFecFieldSize() const {return fec_m;}
+        
+        // returns difference (a - b)
+        INT32 Difference(NormBlockId a, NormBlockId b) const
+            {return NormBlockId::Difference(a, b, fec_block_mask);}
+        // returns -1, 0, or 1 for (a < b), (a == b), or (a > b), respectively
+        int Compare(NormBlockId a, NormBlockId b) const
+            {return NormBlockId::Compare(a, b, fec_block_mask);}
+        void Increment(NormBlockId& b, UINT32 i = 1) const
+            {b.Increment(i, fec_block_mask);}
+        void Decrement(NormBlockId& b, UINT32 i = 1) const
+            {b.Decrement(i, fec_block_mask);}
+        
         bool HaveInfo() const {return (info_len > 0);}
         bool HasInfo() const {return (NULL != info_ptr);}
+        void ClearInfo()
+        {
+            if (NULL != info_ptr) delete[] info_ptr;
+            info_ptr = NULL;
+            info_len = 0;
+            pending_info = false;
+        }
         
         const char* GetInfo() const {return info_ptr;}
         UINT16 GetInfoLength() const {return info_len;}
@@ -94,8 +129,6 @@ class NormObject
         }
         void Close();
         
-        
-        
         virtual bool WriteSegment(NormBlockId   blockId, 
                                   NormSegmentId segmentId, 
                                   const char*   buffer) = 0;
@@ -119,17 +152,19 @@ class NormObject
         NormBlockId GetFinalBlockId() const {return final_block_id;}
         UINT32 GetBlockSize(NormBlockId blockId) const
         {
-            return (((UINT32)blockId < large_block_count) ? large_block_size : 
-                                                            small_block_size);
+            return ((blockId.GetValue() < large_block_count) ? large_block_size : 
+                                                               small_block_size);
         }
         
         NormObjectSize GetBytesPending() const;
         
         bool IsPending(bool flush = true) const;
-        bool IsRepairPending() const;
+        bool IsRepairPending();
         bool IsPendingInfo() {return pending_info;}
         bool PendingMaskIsSet() const
             {return pending_mask.IsSet();}
+        unsigned int GetPendingMaskSize() const
+            {return pending_mask.GetSize();}
         bool GetFirstPending(NormBlockId& blockId) const 
         {
             UINT32 index = 0;
@@ -139,7 +174,7 @@ class NormObject
         }
         bool GetNextPending(NormBlockId& blockId) const
         {
-            UINT32 index = (UINT32)blockId;
+            UINT32 index = blockId.GetValue();
             bool result = pending_mask.GetNextSet(index);
             blockId = NormBlockId(index);
             return result;
@@ -161,7 +196,7 @@ class NormObject
         
         bool GetNextRepair(NormBlockId& blockId) const
         {
-            UINT32 index = (UINT32)blockId;
+            UINT32 index = blockId.GetValue();
             bool result = repair_mask.GetNextSet(index);
             blockId = NormBlockId(index);
             return result;
@@ -174,7 +209,7 @@ class NormObject
             return result;
         }
         
-        bool FindRepairIndex(NormBlockId& blockId, NormSegmentId& segmentId) const;
+        bool FindRepairIndex(NormBlockId& blockId, NormSegmentId& segmentId);
                     
         // Methods available to sender for transmission
         bool NextSenderMsg(NormObjectMsg* msg);
@@ -185,26 +220,17 @@ class NormObject
         void ClearFirstPass() {first_pass = false};*/
         
         bool TxReset(NormBlockId firstBlock = NormBlockId(0), bool requeue = false);
-        bool TxResetBlocks(NormBlockId nextId, NormBlockId lastId);
+        bool TxResetBlocks(const NormBlockId& nextId, const NormBlockId& lastId);
         bool TxUpdateBlock(NormBlock*       theBlock, 
                            NormSegmentId    firstSegmentId, 
                            NormSegmentId    lastSegmentId,
-                           UINT16           numErasures)
-        {
-            NormBlockId blockId = theBlock->GetId();
-            bool result = theBlock->TxUpdate(firstSegmentId, lastSegmentId, 
-                                             GetBlockSize(blockId), nparity, 
-                                             numErasures);
-            ASSERT(result ? pending_mask.Set(blockId) : true);
-            result = result ? pending_mask.Set(blockId) : false;
-            return result; 
-        }  
+                           UINT16           numErasures);
         bool HandleInfoRequest(bool holdoff);
-        bool HandleBlockRequest(NormBlockId nextId, NormBlockId lastId);
+        bool HandleBlockRequest(const NormBlockId& nextId, const NormBlockId& lastId);
         NormBlock* FindBlock(NormBlockId blockId) {return block_buffer.Find(blockId);}
         bool ActivateRepairs();
-        bool IsRepairSet(NormBlockId blockId) {return repair_mask.Test(blockId);}
-        bool IsPendingSet(NormBlockId blockId) {return pending_mask.Test(blockId);}
+        bool IsRepairSet(NormBlockId blockId) {return repair_mask.Test(blockId.GetValue());}
+        bool IsPendingSet(NormBlockId blockId) {return pending_mask.Test(blockId.GetValue());}
         bool AppendRepairAdv(NormCmdRepairAdvMsg& cmd);
         
         NormBlockId GetMaxPendingBlockId() const {return max_pending_block;}
@@ -236,12 +262,12 @@ class NormObject
         bool ReceiverRewindCheck(NormBlockId    blockId,
                                  NormSegmentId  segmentId);
         bool IsRepairPending(bool flush);
-        bool AppendRepairRequest(NormNackMsg& nack, bool flush);
+        bool AppendRepairRequest(NormNackMsg& nack, bool flush, UINT16 payloadMax);
         void SetRepairInfo() {repair_info = true;}
         bool SetRepairs(NormBlockId first, NormBlockId last)
         {
-            return (first == last) ? repair_mask.Set(first) :
-                                     repair_mask.SetBits(first, repair_mask.Delta(last,first)+1); 
+            return ((first == last) ? repair_mask.Set(first.GetValue()) :
+                                      repair_mask.SetBits(first.GetValue(), repair_mask.Difference(last.GetValue(),first.GetValue())+1)); 
         }
         void SetLastNackTime(const ProtoTime& theTime)
             {last_nack_time = theTime;}
@@ -258,7 +284,17 @@ class NormObject
                    const NormObjectId&      objectId); 
     
         void Accept() {accepted = true;}
-    
+
+#ifdef USE_PROTO_TREE    
+        // Proto::Tree item required overrides
+        const char* GetKey() const
+            {return transport_id.GetValuePtr();}
+        unsigned int GetKeysize() const
+            {return (8*sizeof(UINT16));}
+        ProtoTree::Endian GetEndian() const
+            {return ProtoTree::GetNativeEndian();}
+#endif // USE PROTO_TREE
+                
         NormObject::Type      type;
         class NormSession&    session;
         class NormSenderNode* sender;  // NULL value indicates local (tx) object
@@ -269,6 +305,7 @@ class NormObject
         UINT16                segment_size;
         UINT8                 fec_id;
         UINT8                 fec_m;
+        UINT32                fec_block_mask;
         UINT16                ndata;
         UINT16                nparity;
         NormBlockBuffer       block_buffer;
@@ -298,8 +335,9 @@ class NormObject
         bool                  notify_on_update;
         
         const void*           user_data;  // for NORM API usage only
-        
-        NormObject*           next; 
+#ifndef USE_PROTO_TREE       
+        NormObject*           next;
+#endif  
 };  // end class NormObject
 
 
@@ -325,7 +363,7 @@ class NormFileObject : public NormObject
             return result;
         }
         bool PadToSize()
-            {return file.Pad(NormObject::GetSize().GetOffset());}
+            {return file.IsOpen() ? file.Pad(NormObject::GetSize().GetOffset()) : false;}
         
         virtual bool WriteSegment(NormBlockId   blockId, 
                                   NormSegmentId segmentId, 
@@ -349,9 +387,12 @@ class NormDataObject : public NormObject
 {
     // (TBD) allow support of greater than 4GB size data objects
     public:
+        typedef void (*DataFreeFunctionHandle)(char*);
+    
         NormDataObject(class NormSession&       theSession,
                        class NormSenderNode*    theSender,
-                       const NormObjectId&      objectId);
+                       const NormObjectId&      objectId,
+                       DataFreeFunctionHandle   dataFreeFunc);
         ~NormDataObject();
         
         bool Open(char*       dataPtr,
@@ -380,13 +421,17 @@ class NormDataObject : public NormObject
         
         virtual char* RetrieveSegment(NormBlockId   blockId,
                                       NormSegmentId segmentId);
+        
+        
             
     private:
-        NormObjectSize  large_block_length;
-        NormObjectSize  small_block_length;
-        char*           data_ptr;
-        UINT32          data_max;
-        bool            data_released;   // when true, data_ptr is deleted 
+        NormObjectSize          large_block_length;
+        NormObjectSize          small_block_length;
+        char*                   data_ptr;
+        UINT32                  data_max;
+        bool                    data_released;   // when true, data_ptr is deleted 
+        DataFreeFunctionHandle  data_free_func;
+        
                                          // on NormDataObject destruction
 };  // end class NormDataObject
 
@@ -426,7 +471,7 @@ class NormStreamObject : public NormObject
         bool GetPushMode() const {return push_mode;}
         
         bool IsOldBlock(NormBlockId blockId) const
-            {return (!stream_buffer.IsEmpty() && (blockId < stream_buffer.RangeLo()));}
+            {return (!stream_buffer.IsEmpty() && (Compare(blockId, stream_buffer.RangeLo()) < 0));}
 
         bool IsClosing() {return stream_closing;}
         bool HasVacancy() 
@@ -459,6 +504,11 @@ class NormStreamObject : public NormObject
         }
         bool StreamAdvance();
         
+        NormBlockId GetSyncId() const
+            {return stream_sync_id;}
+        NormBlockId GetNextId() const
+            {return stream_next_id;}
+        
         virtual bool WriteSegment(NormBlockId   blockId, 
                                   NormSegmentId segmentId, 
                                   const char*   buffer);
@@ -479,15 +529,15 @@ class NormStreamObject : public NormObject
         
         bool LockSegments(NormBlockId blockId, NormSegmentId firstId, NormSegmentId lastId);
         NormBlockId StreamBufferLo() const {return stream_buffer.RangeLo();} 
+        NormBlockId RepairWindowLo() const;
         void Prune(NormBlockId blockId, bool updateStatus);
         
         bool IsFlushPending() {return flush_pending;}
-        NormBlockId FlushBlockId()
-            {return (write_index.segment ? write_index.block : 
-                     (NormBlockId((UINT32)write_index.block-1)));}
-        NormSegmentId FlushSegmentId()
-            {return (write_index.segment ? (write_index.segment-1) : 
-                                           (ndata-1));}
+        
+        NormBlockId FlushBlockId() const;
+        
+        NormSegmentId FlushSegmentId() const
+            {return (write_index.segment ? (write_index.segment-1) : (ndata-1));}
         
         NormBlockId GetNextBlockId() const
             {return (sender ? read_index.block : write_index.block);}
@@ -529,6 +579,7 @@ class NormStreamObject : public NormObject
             public:
                 NormBlockId     block;
                 NormSegmentId   segment; 
+                UINT16          offset;
         };
         // Extra state for STREAM objects
         bool                        stream_sync;
@@ -551,9 +602,10 @@ class NormStreamObject : public NormObject
         bool                        msg_start;
         FlushMode                   flush_mode;
         bool                        push_mode;
-        
+        bool                        stream_broken;
         bool                        stream_closing;
         
+        
         // For threaded API purposes
         UINT32                      block_pool_threshold;
 };  // end class NormStreamObject
@@ -588,6 +640,10 @@ class NormSimObject : public NormObject
 };  // end class NormSimObject
 #endif // SIMULATE
 
+#ifdef USE_PROTO_TREE
+class NormObjectTree : public ProtoSortedTreeTemplate<NormObject> {};
+#endif // USE_PROTO_TREE
+
 class NormObjectTable
 {
     public:
@@ -600,7 +656,6 @@ class NormObjectTable
         void SetRangeMax(UINT16 rangeMax);
         void Destroy();
         
-        bool IsInited() const {return (NULL != table);}
         UINT16 GetRangeMax() const {return range_max;}
         bool CanInsert(NormObjectId objectId) const;
         bool Insert(NormObject* theObject);
@@ -612,7 +667,23 @@ class NormObjectTable
         bool IsEmpty() const {return (0 == range);}
         UINT32 GetCount() const {return count;}
         const NormObjectSize& GetSize() const {return size;}
+
+#ifdef USE_PROTO_TREE
+        class Iterator
+        {
+            public:
+                Iterator(NormObjectTable& objectTable);
+                NormObject* GetNextObject();
+                NormObject* GetPrevObject();
+                void Reset();
+                
+            private:
+                const NormObjectTable&      table;
+                NormObjectTree::Iterator    iterator;
+                NormObject*                 next_object;
+        }; 
         
+#else        
         class Iterator
         {
             public:
@@ -626,12 +697,19 @@ class NormObjectTable
                 bool                    reset;
                 NormObjectId            index;
         }; 
+#endif // if/else USE_PROTO_TREE
             
     private:
+#ifndef USE_PROTO_TREE
         NormObject* Next(NormObject* o) const {return o->next;}    
-        
+#endif
+    
+#ifdef USE_PROTO_TREE
+        NormObjectTree  tree;
+#else        
         NormObject**    table;
         UINT16          hash_mask;       
+#endif // if/else USE_PROTO_TREE
         UINT16          range_max;  // max range of objects that can be kept
         UINT16          range;      // zero if "object table" is empty
         NormObjectId    range_lo;
