@@ -18,56 +18,57 @@ class ProtoExample :
 #endif  // if/else SIMULATE
                      
 {
-public:
-  ProtoExample();
-  ~ProtoExample();
-  
-  /**
-   * Override from ProtoApp or NsProtoSimAgent base
-   */
-  bool OnStartup(int argc, const char*const* argv);
-  /**
-   * Override from ProtoApp or NsProtoSimAgent base
-   */
-  bool ProcessCommands(int argc, const char*const* argv);
-  /**
-   * Override from ProtoApp or NsProtoSimAgent base
-   */
-  void OnShutdown();
-  /**
-   * Override from ProtoApp or NsProtoSimAgent base
-   */
-  virtual bool HandleMessage(unsigned int len, const char* txBuffer,const ProtoAddress& srcAddr) {return true;}
+    public:
+        ProtoExample();
+        ~ProtoExample();
+
+        /**
+        * Override from ProtoApp or NsProtoSimAgent base
+        */
+        bool OnStartup(int argc, const char*const* argv);
+        /**
+        * Override from ProtoApp or NsProtoSimAgent base
+        */
+        bool ProcessCommands(int argc, const char*const* argv);
+        /**
+        * Override from ProtoApp or NsProtoSimAgent base
+        */
+        void OnShutdown();
+        /**
+        * Override from ProtoApp or NsProtoSimAgent base
+        */
+        virtual bool HandleMessage(unsigned int len, const char* txBuffer,const ProtoAddress& srcAddr) {return true;}
+
+    private:
+        enum CmdType {CMD_INVALID, CMD_ARG, CMD_NOARG};
+        static CmdType GetCmdType(const char* string);
+        bool OnCommand(const char* cmd, const char* val);        
+        void Usage();
+
+        void OnTxTimeout(ProtoTimer& theTimer);
+        void OnUdpSocketEvent(ProtoSocket&       theSocket, 
+                            ProtoSocket::Event theEvent);
+        void OnClientSocketEvent(ProtoSocket&       theSocket, 
+                               ProtoSocket::Event theEvent);
+        void OnServerSocketEvent(ProtoSocket&       theSocket, 
+                               ProtoSocket::Event theEvent);
+        static const char* const CMD_LIST[];
+        static void SignalHandler(int sigNum);
+
+        // ProtoTimer/ UDP socket demo members
+        ProtoTimer          tx_timer;
+        ProtoAddress        dst_addr;
+
+        ProtoSocket         udp_tx_socket;
+        ProtoSocket         udp_rx_socket;
+
+        // TCP socket demo members
+        ProtoSocket         server_socket;
+        ProtoSocket         client_socket;
+        unsigned int        client_msg_count;
+        ProtoSocket::List   connection_list;
+        bool                use_html;
 
-private:
-  enum CmdType {CMD_INVALID, CMD_ARG, CMD_NOARG};
-  static CmdType GetCmdType(const char* string);
-  bool OnCommand(const char* cmd, const char* val);        
-  void Usage();
-  
-  bool OnTxTimeout(ProtoTimer& theTimer);
-  void OnUdpSocketEvent(ProtoSocket&       theSocket, 
-                        ProtoSocket::Event theEvent);
-  void OnClientSocketEvent(ProtoSocket&       theSocket, 
-                           ProtoSocket::Event theEvent);
-  void OnServerSocketEvent(ProtoSocket&       theSocket, 
-                           ProtoSocket::Event theEvent);
-  static const char* const CMD_LIST[];
-  static void SignalHandler(int sigNum);
-  
-  // ProtoTimer/ UDP socket demo members
-  ProtoTimer          tx_timer;
-  ProtoAddress        dst_addr;
-  
-  ProtoSocket         udp_tx_socket;
-  ProtoSocket         udp_rx_socket;
-  
-  // TCP socket demo members
-  ProtoSocket         server_socket;
-  ProtoSocket         client_socket;
-  unsigned int        client_msg_count;
-  ProtoSocket::List   connection_list;
-  bool                use_html;
 }; // end class ProtoExample
 
 
@@ -183,10 +184,18 @@ bool ProtoExample::OnStartup(int argc, const char*const* argv)
     }  
     
 #ifndef SIMULATE     
+
     // Here's some code to test the ProtoSocket routines for network interface info       
     ProtoAddress localAddress;
     char nameBuffer[256];
     nameBuffer[255] = '\0';
+
+    // ljt test
+    ProtoAddress dst;
+    dst.ResolveFromString("fe80::426c:8fff:fe31:4f8e");
+    if (dst.ResolveToName(nameBuffer,255))
+        TRACE("protoExample: LJT ipv6 host name: %s\n", nameBuffer);
+
     
     if (localAddress.ResolveLocalAddress())
     {
@@ -194,7 +203,7 @@ bool ProtoExample::OnStartup(int argc, const char*const* argv)
         if (localAddress.ResolveToName(nameBuffer, 255))
             TRACE("protoExample: local default host name: %s\n", nameBuffer);
         else
-            TRACE("protoExample: unable to resolve local default IP address\n");
+            TRACE("protoExample: unable to resolve local default IP address to name\n");
     }      
     else
     {
@@ -242,11 +251,7 @@ bool ProtoExample::OnStartup(int argc, const char*const* argv)
                 ProtoAddressList::Iterator iterator(addrList);
                 while (iterator.GetNextAddress(ifaceAddr))
                 {
-#ifdef WIN32
-					TRACE(" %s", ifaceAddr.GetHostString());
-#else
-                    TRACE(" %s/%d", ifaceAddr.GetHostString(), ProtoNet::GetInterfaceAddressMask(nameBuffer, ifaceAddr));
-#endif // if/else WIN32
+					TRACE(" %s/%d", ifaceAddr.GetHostString(),ProtoNet::GetInterfaceAddressMask(nameBuffer, ifaceAddr));
                     if (ifaceAddr.IsLinkLocal()) TRACE(" (link local)");
                 }
                 TRACE("\n");
@@ -259,15 +264,53 @@ bool ProtoExample::OnStartup(int argc, const char*const* argv)
                 ProtoAddressList::Iterator iterator(addrList);
                 while (iterator.GetNextAddress(ifaceAddr))
                 {
-#ifdef WIN32
-					TRACE(" %s", ifaceAddr.GetHostString());
-#else
+
                     TRACE(" %s/%d", ifaceAddr.GetHostString(), ProtoNet::GetInterfaceAddressMask(nameBuffer, ifaceAddr));
-#endif // if/else WIN32
-                    if (ifaceAddr.IsLinkLocal()) TRACE(" (link local)");
+
+					if (ifaceAddr.IsLinkLocal()) TRACE(" (link local)");
                 }
                 TRACE("\n");
             }
+            addrList.Destroy();
+
+			// Test our interface add routine
+            /*
+			char * ifaceName = "192.168.1.6";
+			ProtoAddress tmpAddr;
+			tmpAddr.ConvertFromString("192.168.1.6");
+			int maskLen = 24;
+			bool result = ProtoNet::AddInterfaceAddress(ifaceName, tmpAddr, maskLen);
+            */
+
+
+#ifndef WIN32
+            // This code should work for Linux and BSD (incl. Mac OSX)
+            // Get IPv4 group memberships for interfaces
+            if (ProtoNet::GetGroupMemberships(nameBuffer, ProtoAddress::IPv4, addrList))
+            {
+                TRACE("          IPv4 memberships:");
+                ProtoAddressList::Iterator iterator(addrList);
+                ProtoAddress groupAddr;
+                while (iterator.GetNextAddress(groupAddr))
+                {
+                    TRACE(" %s", groupAddr.GetHostString());
+                }       
+                TRACE("\n");
+            }
+            addrList.Destroy();
+            if (ProtoNet::GetGroupMemberships(nameBuffer, ProtoAddress::IPv6, addrList))
+            {
+                TRACE("          IPv6 memberships:");
+                ProtoAddressList::Iterator iterator(addrList);
+                ProtoAddress groupAddr;
+                while (iterator.GetNextAddress(groupAddr))
+                {
+                    TRACE(" %s", groupAddr.GetHostString());
+                }       
+                TRACE("\n");
+            }
+            addrList.Destroy();
+#endif // WIN32
         }
         delete[] indexArray;
     }
@@ -275,8 +318,6 @@ bool ProtoExample::OnStartup(int argc, const char*const* argv)
     {
         TRACE("protoExample: host has no network interfaces?!\n");
     }  
-    
-    
     // Here's some code to get the system routing table
     ProtoRouteTable routeTable;
     ProtoRouteMgr* routeMgr = ProtoRouteMgr::Create();
@@ -308,7 +349,7 @@ bool ProtoExample::OnStartup(int argc, const char*const* argv)
                 entry->GetMetric());
     }
     //ProtoAddress dst;
-    //dst.ResolveFromString("132.250.93.34");
+    //dst.ResolveFromString("10.1.2.3");
     //ProtoAddress gw;
     //routeMgr->SetRoute(dst, 32, gw, 4, 0);
     routeMgr->Close();
@@ -489,12 +530,14 @@ bool ProtoExample::OnCommand(const char* cmd, const char* val)
                 return false;
             }
             server.SetPort(atoi(ptr));
+            TRACE("calling connect ...\n");
             if (!client_socket.Connect(server))
             {
                 PLOG(PL_ERROR, "ProtoExample::ProcessCommand(connect) error connecting\n");
                 return false;
             }
             client_msg_count = 5;  // (Send 5 messages, then disconnect)
+            TRACE("   (completed. starting output notification ...\n");
             client_socket.StartOutputNotification();
         }
         else
@@ -531,7 +574,7 @@ bool ProtoExample::OnCommand(const char* cmd, const char* val)
     return true;
 }  // end ProtoExample::OnCommand()
 
-bool ProtoExample::OnTxTimeout(ProtoTimer& /*theTimer*/)
+void ProtoExample::OnTxTimeout(ProtoTimer& /*theTimer*/)
 {
     const char* string = "Hello there UDP peer, how are you doing?";
     unsigned int len = strlen(string) + 1;
@@ -544,10 +587,10 @@ bool ProtoExample::OnTxTimeout(ProtoTimer& /*theTimer*/)
     }
     else if (len != numBytes)
     {
-        PLOG(PL_ERROR, "ProtoExample::OnClientSocketEvent() incomplete SendTo()\n");                
+        PLOG(PL_ERROR, "ProtoExample::OnTxTimeout() incomplete SendTo()\n");                
         TRACE("   (only sent %lu of %lu bytes)\n", numBytes, len);
     }
-    return true;
+
 }  // end ProtoExample::OnTxTimeout()
 
 void ProtoExample::OnUdpSocketEvent(ProtoSocket&       theSocket, 
@@ -654,9 +697,9 @@ void ProtoExample::OnClientSocketEvent(ProtoSocket&       theSocket,
             }
             else if (len != numBytes)
             {
-                PLOG(PL_ERROR, "ProtoExample::OnClientSocketEvent() incomplete Send()\n");                
-                TRACE("   (only sent %lu of %lu bytes)\n", numBytes, len);
-            } 
+                PLOG(PL_WARN, "ProtoExample::OnClientSocketEvent() incomplete Send()\n");                
+				TRACE("   (only sent %lu of %lu bytes (msgCount:%d))\n", numBytes, len, client_msg_count);
+			} 
             else
             {
                 TRACE("sent %u bytes to ProtoServer ...\n", len);
@@ -724,13 +767,13 @@ void ProtoExample::OnServerSocketEvent(ProtoSocket&       theSocket,
         {
             TRACE("ACCEPT) ...\n");
             ProtoSocket* connectedSocket = new ProtoSocket(ProtoSocket::TCP);
-			connectedSocket->SetListener(this, &ProtoExample::OnServerSocketEvent); 
-
-            if (!connectedSocket)
+			if (NULL == connectedSocket)
             {
                 PLOG(PL_ERROR, "ProtoExample::OnServerSocketEvent(ACCEPT) new ProtoSocket error\n");
                 break;
             }
+            connectedSocket->SetListener(this, &ProtoExample::OnServerSocketEvent); 
+
             if (!server_socket.Accept(connectedSocket))
             {
                 PLOG(PL_ERROR, "ProtoExample::OnServerSocketEvent(ACCEPT) error accepting connection\n");
