@@ -1,8 +1,9 @@
 #ifndef _PROTO_GRAPH
 #define _PROTO_GRAPH
 
-#include <protoTree.h>
-#include <protoDebug.h>
+#include "protoTree.h"
+#include "protoDebug.h"
+#include "protoDefs.h"
 
 /**
  * @class ProtoGraph
