@@ -1,9 +1,9 @@
 #ifndef _PROTO_PKT_IP
 #define _PROTO_PKT_IP
 
-#include <protoPkt.h>
-#include <protoAddress.h>
-#include <protoDebug.h>
+#include "protoPkt.h"
+#include "protoAddress.h"
+#include "protoDebug.h"
 
 /**
  * @class ProtoPktIP
@@ -163,6 +163,7 @@ class ProtoPktIPv4 : public ProtoPktIP
         
         enum Flag
         {
+            FLAG_NONE     = 0x00,
             FLAG_RESERVED = 0x80,  // reserved bit
             FLAG_DF       = 0x40,  // 0 = may fragment,  1 = don't fragment 
             FLAG_MF       = 0x20   // 0 = last fragment, 1 = more fragments
@@ -316,7 +317,6 @@ class ProtoPktIPv4 : public ProtoPktIP
         void GetDstAddr(ProtoAddress& addr) const
             {addr.SetRawHostAddress(ProtoAddress::IPv4, (char*)(buffer_ptr+OFFSET_DST_ADDR), 4);}
         
-        
         /// Helper method to get pointer to ID portion of IPv4 header
         const char* GetIDPtr() const
             {return ((char*)buffer_ptr + (OFFSET_ID*2));}
@@ -399,6 +399,8 @@ class ProtoPktIPv4 : public ProtoPktIP
         
         // Return checksum in host byte order
         UINT16 CalculateChecksum(bool set = true);
+        UINT16 FinalizeChecksum()
+            {return CalculateChecksum(true);}
            
     private:
         void SetHeaderLength(UINT8 hdrBytes) 
