@@ -21,6 +21,7 @@
 *
 */
 
+#include "protoDefs.h"
 #include "protoTree.h"
 #include "protoDebug.h"
 
@@ -37,6 +38,9 @@ class ProtoQueue
         bool IsInOtherQueue(Item& item)
             {return item.IsInOtherQueue(*this);}
         
+        bool Contains(Item& item) const
+            {return (NULL != item.GetContainer(*this));}
+        
         // Derived classes MUST implement the
         // Empty() method _and_ call it in
         // their destructor!
@@ -137,6 +141,14 @@ class ProtoQueue
             
                 bool IsInQueue() const
                     {return (!container_list.IsEmpty());}
+            
+                Container* GetContainer(const ProtoQueue& queue) const
+                {
+                    const ProtoQueue* ptr = &queue;
+                    ProtoQueue::Container::Entry* entry = 
+                        static_cast<ProtoQueue::Container::Entry*>(container_list.Find((const char*)&ptr, sizeof(ProtoQueue*) << 3));
+                    return ((NULL != entry) ? &entry->GetContainer() : NULL);
+                } 
                 
             protected:
                 Item();
@@ -153,13 +165,6 @@ class ProtoQueue
             private:
                 bool IsInOtherQueue(const ProtoQueue& queue);
             
-                Container* GetContainer(const ProtoQueue& queue) const
-                {
-                    const ProtoQueue* ptr = &queue;
-                    ProtoQueue::Container::Entry* entry = 
-                        static_cast<ProtoQueue::Container::Entry*>(container_list.Find((const char*)&ptr, sizeof(ProtoQueue*) << 3));
-                    return ((NULL != entry) ? &entry->GetContainer() : NULL);
-                } 
                 void Reference(ProtoQueue::Container& container)
                     {container_list.Insert(container.AccessEntry());}
                 void Dereference(ProtoQueue::Container& container)
@@ -173,7 +178,7 @@ class ProtoQueue
                 ProtoTree       container_list;
             
         };  // end class ProtoQueue::Item
-
+        
     
         /**
          * @class ProtoQueue::ContainerPool
@@ -217,7 +222,6 @@ class ProtoQueue
 
             Container* GetContainer(const Item& item) const
                 {return item.GetContainer(*this);}
-
             
             Container* GetContainerFromPool()
                 {return ((NULL != container_pool) ? container_pool->Get() : NULL);}
@@ -260,6 +264,8 @@ class ProtoSimpleQueue : public ProtoQueue
         
         bool Prepend(Item& theItem);
         bool Append(Item& theItem);
+        bool Insert(Item& theItem, Item& nextItem);  // insert "theItem" before "nextItem"
+        bool InsertAfter(Item& theItem, Item& prevItem);  // insert "theItem" after "prevItem"
         void Remove(Item& theItem);
         
         Item* GetHead() const
@@ -275,6 +281,20 @@ class ProtoSimpleQueue : public ProtoQueue
         }
         Item* RemoveTail();
         
+        Item* GetPrev(Item& item) const
+        {
+            const Container* container = static_cast<Container*>(GetContainer(item));
+            if (NULL != container)
+            {
+                container = static_cast<const Container*>(container->GetPrev());
+                return ((NULL != container) ? container->GetItem() : NULL);
+            }
+            else
+            {
+                return NULL;
+            }
+        }
+        
         bool IsEmpty() const
             {return (NULL != GetHead()) ? false : true;}
         
@@ -283,8 +303,8 @@ class ProtoSimpleQueue : public ProtoQueue
         void Destroy();  // empties queue, deleting items
         
         // TBD - add set operations
-        
-        /*const bool Union(ProtoSimpleQueue &unionQueue, const ProtoSimpleQueue &bQueue);
+        /*
+        const bool Union(ProtoSimpleQueue &unionQueue, const ProtoSimpleQueue &bQueue);
         const bool Intersection(ProtoSimpleQueue &intersectionQueue, const ProtoSimpleQueue &bQueue);
         const bool RelativeComplement(ProtoSimpleQueue &rcQueue, const ProtoSimpleQueue &bQueue);//(a - intersection of ab)
         const bool SymmetricDifference(ProtoSimpleQueue &rcQueue, const ProtoSimpleQueue &bQueue);//(union of relative complements)
@@ -324,24 +344,23 @@ class ProtoSimpleQueue : public ProtoQueue
                     Container* nextContainer = static_cast<Container*>(ProtoList::Iterator::PeekPrevItem());
                     return ((NULL != nextContainer) ? nextContainer->GetItem() : NULL);
                 }
-        };  // end class ProtoSimpleQueueTemplate::Iterator  
+        };  // end class ProtoSimpleQueue::Iterator  
         
         class Container : public ProtoQueue::Container, public ProtoList::Item
         {
            public:
                 Container();
                 ~Container();     
-        };  // end class ProtoSimpleQueueTemplate::Container
+        };  // end class ProtoSimpleQueue::Container
         
         class ContainerPool : public ProtoQueue::ContainerPool
         {
             public:
-                ContainerPool();
-                virtual ~ContainerPool();
-                
+                void Put(Container& theContainer)
+                    {ProtoQueue::ContainerPool::Put(theContainer);}
                 Container* Get()
                     {return static_cast<Container*>(ProtoQueue::ContainerPool::Get());}
-        };  // end class ProtoSimpleQueueTemplate::ContainerPool  
+        };  // end class ProtoSimpleQueue::ContainerPool  
         
     private:
         // TBD - do we want to make CreateContainer() virtual so derived classes can do more?
@@ -373,6 +392,8 @@ class ProtoSimpleQueueTemplate : public ProtoSimpleQueue
             {return static_cast<ITEM_TYPE*>(ProtoSimpleQueue::GetTail());}
         ITEM_TYPE* RemoveTail()
             {return static_cast<ITEM_TYPE*>(ProtoSimpleQueue::RemoveTail());}
+        ITEM_TYPE* GetPrev(ITEM_TYPE& item)
+            {return static_cast<ITEM_TYPE*>(ProtoSimpleQueue::GetPrev(item));}
         
         class Iterator : public ProtoSimpleQueue::Iterator
         {
@@ -475,6 +496,15 @@ class ProtoIndexedQueue : public ProtoQueue
                 unsigned int GetKeysize() const;
                 
         };  // end class ProtoIndexedQueue::Container  
+        
+        class ContainerPool : public ProtoQueue::ContainerPool
+        {
+            public:
+                void Put(Container& theContainer)
+                    {ProtoQueue::ContainerPool::Put(theContainer);}
+                Container* Get()
+                    {return static_cast<Container*>(ProtoQueue::ContainerPool::Get());}
+        };  // end class ProtoIndexedQueue::ContainerPool  
             
         class Iterator : public ProtoTree::Iterator
         {
@@ -482,8 +512,10 @@ class ProtoIndexedQueue : public ProtoQueue
                 Iterator(ProtoIndexedQueue& theQueue, bool reverse = false);
                 virtual ~Iterator();
                 
-                void Reset(bool reverse = false)
-                    {ProtoTree::Iterator::Reset(reverse);}
+                void Reset(bool reverse = false,
+                           const char*  prefix = NULL,
+                           unsigned int prefixSize = 0)
+                    {ProtoTree::Iterator::Reset(reverse, prefix, prefixSize);}
                 
                 Item* GetNextItem()
                 {
@@ -532,10 +564,11 @@ class ProtoIndexedQueueTemplate : public ProtoIndexedQueue
         virtual unsigned int GetKeysize(const Item& item) const = 0;
         
         // Insert the "item" into the tree (will fail if item with equivalent key already in tree)
-        bool Insert(ITEM_TYPE& item)
-            {return ProtoIndexedQueue::Insert(item);}
+        //bool Insert(ITEM_TYPE& item)
+        //    {return ProtoIndexedQueue::Insert(item);}
         
         // Remove the "item" from the tree
+	// LJT THIS HAD BEEN COMMENTED OUT
         void Remove(ITEM_TYPE& item)
             {return ProtoIndexedQueue::Remove(item);}
         
@@ -560,8 +593,10 @@ class ProtoIndexedQueueTemplate : public ProtoIndexedQueue
                  : ProtoIndexedQueue::Iterator(theQueue, reverse) {}
                 ~Iterator() {}
                 
-                void Reset(bool reverse = false)
-                    {ProtoIndexedQueue::Iterator::Reset(reverse);}
+                void Reset(bool         reverse = false,
+                           const char*  prefix = NULL,
+                           unsigned int prefixSize = 0)
+                    {ProtoIndexedQueue::Iterator::Reset(reverse, prefix, prefixSize);}
                 
                 ITEM_TYPE* GetNextItem()
                     {return static_cast<ITEM_TYPE*>(ProtoIndexedQueue::Iterator::GetNextItem());}
@@ -578,6 +613,9 @@ class ProtoIndexedQueueTemplate : public ProtoIndexedQueue
             : ProtoIndexedQueue(usePool) {}
         ProtoIndexedQueueTemplate(ContainerPool* containerPool)
             : ProtoIndexedQueue(containerPool) {}
+        
+    private:
+        using ProtoIndexedQueue::Remove;   // gets rid of hidden overloaded virtual function warning
            
 };  // end class ProtoIndexedQueueTemplate  
     
@@ -621,7 +659,7 @@ class ProtoSortedQueue : public ProtoQueue
     public:
         virtual ~ProtoSortedQueue();
     
-        // Insert the "item" into the tree (will fail if item with equivalent key already in tree)
+        // Insert the "item" into the tree (multiple items with same key is OK)
         bool Insert(Item& item);
         
         // Remove the "item" from the tree
@@ -630,13 +668,25 @@ class ProtoSortedQueue : public ProtoQueue
         bool IsEmpty() const
             {return item_tree.IsEmpty();}
         
-        // Find item with exact match to "key" and "keysize" (keysize is in bits)
+        // Find first item with exact match to "key" and "keysize" (keysize is in bits)
         Item* Find(const char* key, unsigned int keysize) const
         {
             Container* container = item_tree.Find(key, keysize);
             return ((NULL != container) ? container->GetItem() : NULL);
         }
         
+        Item* GetHead() const
+        {
+            Container* head = item_tree.GetHead();
+            return ((NULL != head) ? head->GetItem() : NULL);
+        }
+        
+        Item* GetTail() const
+        {
+            Container* tail = item_tree.GetTail();
+            return ((NULL != tail) ? tail->GetItem() : NULL);
+        }
+        
         void Empty();  // empties queue, but doesn't delete items
         
         void Destroy();  // empties queue, deleting items
@@ -669,6 +719,15 @@ class ProtoSortedQueue : public ProtoQueue
                 
         };  // end class ProtoSortedQueue::Container  
         
+        class ContainerPool : public ProtoQueue::ContainerPool
+        {
+            public:
+                void Put(Container& theContainer)
+                    {ProtoQueue::ContainerPool::Put(theContainer);}
+                Container* Get()
+                    {return static_cast<Container*>(ProtoQueue::ContainerPool::Get());}
+        };  // end class ProtoSortedQueue::ContainerPool  
+        
         class Iterator : public ProtoSortedTree::Iterator
         {
             public:
@@ -683,6 +742,12 @@ class ProtoSortedQueue : public ProtoQueue
                            unsigned int keysize = 0)
                     {ProtoSortedTree::Iterator::Reset(reverse, keyMin, keysize);}
                 
+                void SetCursor(ProtoSortedQueue& theQueue, ProtoQueue::Item& theItem)
+                {
+                    Container* container = static_cast<ProtoSortedQueue::Container*>(theItem.GetContainer(theQueue));
+                    ProtoSortedTree::Iterator::SetCursor(container);
+                }
+                
                 Item* GetNextItem()
                 {
                     Container* nextContainer = static_cast<Container*>(ProtoSortedTree::Iterator::GetNextItem());
@@ -743,12 +808,17 @@ class ProtoSortedQueueTemplate : public ProtoSortedQueue
         
         // Remove the "item" from the tree
         void Remove(ITEM_TYPE& item)
-            {ProtoSortedQueue::Remove(item);}
+            {ProtoSortedQueue::Remove(static_cast<Item&>(item));}
         
-        // Find item with exact match to "key" and "keysize" (keysize is in bits)
+        // Find firat item with exact match to "key" and "keysize" (keysize is in bits)
         ITEM_TYPE* Find(const char* key, unsigned int keysize) const
             {return static_cast<ITEM_TYPE*>(ProtoSortedQueue::Find(key, keysize));}
         
+        ITEM_TYPE* GetHead() const
+            {return static_cast<ITEM_TYPE*>(ProtoSortedQueue::GetHead());}
+        ITEM_TYPE* GetTail() const
+            {return static_cast<ITEM_TYPE*>(ProtoSortedQueue::GetTail());}
+        
         class Iterator : public ProtoSortedQueue::Iterator
         {
             public:
@@ -775,6 +845,9 @@ class ProtoSortedQueueTemplate : public ProtoSortedQueue
             : ProtoSortedQueue(usePool) {}
         ProtoSortedQueueTemplate(ContainerPool* containerPool)
             : ProtoSortedQueue(containerPool) {}
-           
+    
+    private:
+        using ProtoSortedQueue::Remove;   // gets rid of hidden overloaded virtual function warning   
+        
 };  // end class ProtoSortedQueueTemplate          
 #endif // _PROTO_QUEUE
