@@ -2,8 +2,8 @@
 #define _PROTO_SOCKET
 
 #include "protoAddress.h"
-
 #include "protoDebug.h"  // temp
+#include "protoNotify.h"
 
 #ifdef WIN32
 /*#ifdef _WIN32_WCE
@@ -23,6 +23,11 @@ typedef GUID *LPGUID;
 #include <unistd.h>  // for read()
 #include <stdio.h>
 #endif // if/else WIN32/UNIX
+// "RAW" is predefined IOCTL macro in Gnu/Hurd Linux
+// so this deconflicts it for the ProtoSocket::Protocol enum
+#ifdef RAW
+#undef RAW
+#endif // RAW
 /**
  * @class ProtoSocket
  *
@@ -37,7 +42,7 @@ typedef GUID *LPGUID;
  * UDP or TCP socket.
  */
 
-class ProtoSocket
+class ProtoSocket : public ProtoNotify
 {
     public:
 			// Type definitions
@@ -61,6 +66,13 @@ class ProtoSocket
 				ECN_CE   = 0x03   // ECN "Congestion Experienced" (old CE)
 		}; 
 
+        enum IPv6SupportStatus
+        {
+            IPV6_UNKNOWN,
+            IPV6_UNSUPPORTED,
+            IPV6_SUPPORTED
+        };
+
 #ifdef SIMULATE
 		class Proxy {};
 		typedef Proxy* Handle;
@@ -132,9 +144,9 @@ class ProtoSocket
         // These are valid for connected sockets only
         const ProtoAddress& GetSourceAddr() const {return source_addr;}
 		const ProtoAddress& GetDestination() const {return destination;}
-		// I.T. to set the destination on a socket after an ACCEPT.
-		void SetDestination(const ProtoAddress& destin) 
-            {destination=destin;}
+		// I.T. helper method to set the destination on a socket after an ACCEPT.
+		void SetDestination(const ProtoAddress& theDestination) 
+            {destination = theDestination;}
 #endif  // OPNET
 #ifdef WIN32
 		HANDLE GetInputEventHandle() {return input_event_handle;}
@@ -161,7 +173,7 @@ class ProtoSocket
 		}
 
 		// Read/Write methods
-		bool SendTo(const char* buffer, unsigned int buflen, const ProtoAddress& dstAddr);
+		bool SendTo(const char* buffer, unsigned int &buflen, const ProtoAddress& dstAddr);
 		bool RecvFrom(char* buffer, unsigned int& numBytes, ProtoAddress& srcAddr);
         bool RecvFrom(char* buffer, unsigned int& numBytes, ProtoAddress& srcAddr, ProtoAddress& dstAddr); 
 		bool Send(const char* buffer, unsigned int& numBytes);
@@ -170,7 +182,7 @@ class ProtoSocket
 		// This was for debugging?? Remove??
         bool Read(char* buffer, unsigned int &numBytes)
         {
-            int result = read(handle, buffer, numBytes);
+            ssize_t result = read(handle, buffer, numBytes);
             if (result < 0)
             {
                 perror("read() error");
@@ -185,13 +197,14 @@ class ProtoSocket
             }   
             else
             {
-                numBytes = result;
+                numBytes = (unsigned int)result;
                 return true;
             }
         }
 #endif // !WIN32
 		// Attributes
 		bool SetTTL(unsigned char ttl);
+		bool SetUnicastTTL(unsigned char ttl);
 		bool SetLoopback(bool loopback);
 		bool SetBroadcast(bool broadcast);
         bool SetFragmentation(bool enable);
@@ -209,9 +222,10 @@ class ProtoSocket
 
 		// Helper methods
 #ifdef HAVE_IPV6
-		static bool HostIsIPv6Capable();
-		static bool SetHostIPv6Capable();
-		bool SetFlowLabel(UINT32 label);
+	static bool HostIsIPv6Capable();
+        // Temporarily retained for backward compatability
+	static bool SetHostIPv6Capable() {return true;}
+	bool SetFlowLabel(UINT32 label);
 #endif //HAVE_IPV6
 
         // These are some static network "helper" functions that get information
@@ -222,9 +236,9 @@ class ProtoSocket
         // the implementations here will eventually be removed _and_ the ProtoSocket
         // static method declarations themselves will be eventually deprecated  
         //          
-		// This appends addresses of type "addrType" to the "addrList"
-		static bool GetHostAddressList(ProtoAddress::Type  addrType,
-								       ProtoAddressList&   addrList);
+	// This appends addresses of type "addrType" to the "addrList"
+	static bool GetHostAddressList(ProtoAddress::Type  addrType,
+				       ProtoAddressList&   addrList);
 
         static bool GetInterfaceAddressList(const char*         ifName, 
 				                            ProtoAddress::Type  addrType,
@@ -242,69 +256,30 @@ class ProtoSocket
         static bool GetInterfaceName(unsigned int index, char* buffer, unsigned int buflen);   
         static bool GetInterfaceName(const ProtoAddress& ifAddr, char* buffer, unsigned int buflen); 
         
-        // Asynchronous I/O notification stuff
-        enum Flag
-        {
-            NOTIFY_NONE      =   0x00,
-            NOTIFY_INPUT     =   0x01,
-            NOTIFY_OUTPUT    =   0x02,
-            NOTIFY_EXCEPTION =  0x04,
-	        NOTIFY_ERROR     =   0x08
-        };
         class Notifier
         {
             public:
                 virtual ~Notifier() {}
                 virtual bool UpdateSocketNotification(ProtoSocket& theSocket, 
-                                                      int          notifyFlags) {return true;} 
+                                                      int          notifyFlags) {return true;}
         };
-        Notifier* GetNotifier() const {return notifier;}
         bool SetNotifier(ProtoSocket::Notifier* theNotifier);
-        bool StartOutputNotification()
-        {
-            notify_output = true;
-            notify_output = UpdateNotification();
-#ifdef WIN32
-            output_ready = true;
-#endif // WIN32
-            return notify_output;   
-        }
-        void StopOutputNotification()
-        {
-            notify_output = false;
-            UpdateNotification();    
-        }  
-        bool NotifyOutput() {return notify_output;}
+		Notifier* GetNotifier() const {return notifier;}
+        
+        bool StartInputNotification();
+        void StopInputNotification();
+        bool InputNotification() const
+            {return notify_input;}
+		bool StartOutputNotification();
+        void StopOutputNotification();
+        bool OutputNotification() const
+            {return notify_output;}
+        bool StartExceptionNotification();
+        void StopExceptionNotification();
+        bool ExceptionNotification() const
+            {return notify_exception;}
 
-        bool StartInputNotification()
-        {
-            notify_input = true;
-            notify_input = UpdateNotification();
-#ifdef WIN32
-            input_ready = true; 
-#endif // WIN32
-            return notify_input;   
-        }
-        void StopInputNotification()
-        {
-            notify_input = false;
-            UpdateNotification();    
-        }  
-        bool NotifyInput() {return notify_input;}
-
-        bool StartExceptionNotification()
-        {
-            notify_exception = true;
-            notify_exception = UpdateNotification();
-            return notify_exception;
-        }
-        void StopExceptionNotification()
-        {   
-            notify_exception = false;
-            UpdateNotification();
-        }
-
-        void OnNotify(ProtoSocket::Flag theFlag);
+        void OnNotify(ProtoNotify::NotifyFlag theFlag);
         
         enum Event {INVALID_EVENT, CONNECT, ACCEPT, SEND, RECV, DISCONNECT, ERROR_, EXCEPTION};
         
@@ -315,14 +290,14 @@ class ProtoSocket
         template <class listenerType>
         bool SetListener(listenerType* theListener, void(listenerType::*eventHandler)(ProtoSocket&, Event))
         {
-            bool doUpdate = ((NULL == listener) && (NULL != theListener)) || 
-                            ((NULL == theListener) && (NULL != listener));
-            if (listener) delete listener;
-            listener = theListener ? new LISTENER_TYPE<listenerType>(theListener, eventHandler) : NULL;
-            bool result = theListener ? (NULL != theListener) : true;
+            bool doUpdate = ((NULL != theListener) || (NULL != listener));
+            if (NULL != listener) delete listener;
+            listener = (NULL != theListener) ? new LISTENER_TYPE<listenerType>(theListener, eventHandler) : NULL;
+            bool result = (NULL != theListener) ? (NULL != listener) : true;
             return result ? (doUpdate ? UpdateNotification() : true) : false;
         }
         
+        
         bool HasListener() 
             {return (NULL != listener);}
         void SetUserData(const void* userData) {user_data = userData;}
@@ -431,7 +406,7 @@ class ProtoSocket
                 LISTENER_TYPE(listenerType* theListener, 
                               void(listenerType::*eventHandler)(ProtoSocket&, Event))
                     : listener(theListener), event_handler(eventHandler) {}
-                void on_event(ProtoSocket& theSocket, Event theEvent)                    
+                void on_event(ProtoSocket& theSocket, Event theEvent) 
                     {(listener->*event_handler)(theSocket, theEvent);}
                 Listener* duplicate()
                     {return (static_cast<Listener*>(new LISTENER_TYPE<listenerType>(listener, event_handler)));}
@@ -467,8 +442,8 @@ class ProtoSocket
 #ifdef WIN32
         HANDLE                  input_event_handle;
         HANDLE                  output_event_handle;
-        bool                    output_ready;
-        bool                    input_ready;
+        bool                    output_ready;  // used to morph edge triggered Win32 sockets to level-triggered behavior
+        bool                    input_ready;   // used to morph edge triggered Win32 sockets to level-triggered behavior
         bool                    closing;
 #endif // WIN32
         Listener*               listener;   
@@ -484,6 +459,8 @@ class ProtoSocket
 		static int GetInterfaceList(struct ifconf& conf);  // helper fn
 #endif  // if/else WIN32
 
+        static IPv6SupportStatus ipv6_support_status;
+
 };  // end class ProtoSocket
 
 #endif // _PROTO_SOCKET
