@@ -44,10 +44,18 @@ ProtoAddress::ProtoAddress()
 {
     memset(&addr, 0, sizeof(addr));
 }
+
 ProtoAddress::ProtoAddress(const ProtoAddress& theAddr)
 {
     *this = theAddr;
 }
+
+
+ProtoAddress::ProtoAddress(const char* theAddr)
+{
+    ConvertFromString(theAddr);
+}
+
 ProtoAddress::~ProtoAddress()
 {
 }
@@ -330,13 +338,18 @@ const char* ProtoAddress::GetHostString(char* buffer, unsigned int buflen) const
                 char* ptr = strchr(buffer, '[');  // nuke start bracket
                 if (ptr)
                 {
+					char * pch;
                     size_t len = strlen(buffer);
                     if (len > buflen) len = buflen;
-                    memmove(buffer, ptr, len - (ptr-buffer));   
-                }   
+					ptr++;
+					memmove(buffer, ptr, len - (ptr-buffer));   
+                }  
                 ptr = strrchr(buffer, '%');  // nuke if index, if applicable
                 if (!ptr) ptr = strrchr(buffer, ']'); // nuke end bracket
-                if (ptr) ptr = '\0';
+				ptr = (char *)memchr(buffer, ']', strlen(buffer));
+				if (ptr)
+					buffer[ptr - buffer] = '\0';
+
             }
 #endif // HAVE_IPV6
             return buffer ? buffer : "(null)";
@@ -670,6 +683,29 @@ unsigned int ProtoAddress::SetCommonTail(const ProtoAddress& theAddr)
     return GetLength();
 }  // end ProtoAddress::SetCommonTail()
 
+
+bool ProtoAddress::PrefixIsEqual(const ProtoAddress& theAddr, UINT8 prefixLen) const
+{
+    // Compare address "type" and "prefixLen" bits of address
+    if (!IsValid() && !theAddr.IsValid()) return true;
+    if (type == theAddr.type)
+    {
+        const char* ptr1 = GetRawHostAddress();
+        const char* ptr2 = theAddr.GetRawHostAddress();
+        size_t nbyte = prefixLen >> 3;
+        if ((0 == nbyte) || (0 == memcmp(ptr1, ptr2, nbyte)))
+        {
+            UINT8 nbit = prefixLen & 0x07;
+            if (0 == nbit) return true;
+            char mask = 0xff << (8 - nbit);
+            if ((mask & ptr1[nbyte]) == (mask & ptr2[nbyte]))
+                return true;
+        }
+    }
+    return false;
+}  // end ProtoAddress::PrefixIsEqual()
+
+
 UINT8 ProtoAddress::GetPrefixLength() const
 {
     UINT8* ptr = NULL;
@@ -1281,7 +1317,8 @@ bool ProtoAddress::ResolveFromString(const char* text)
         return false;
 #endif // if/else WIN32/UNIX
     }
-#endif  // HAVE_IPV6 
+#endif  // HAVE_IPV6
+#if defined(WIN32) || !defined(HAVE_IPV6)
     // Use this approach as a backup for NT-4 or if !HAVE_IPV6
     // 1) is it a "dotted decimal" address?
     struct sockaddr_in* addrPtr = (struct sockaddr_in*)&addr;
@@ -1323,9 +1360,11 @@ bool ProtoAddress::ResolveFromString(const char* text)
         PLOG(PL_ERROR, "ProtoAddress::ResolveFromString gethostbyname() returned unsupported address family!\n");
         return false;
     }
+#endif // WIN32 || !HAVE_IPV6
 #endif // !SIMULATE
+    
 }  // end ProtoAddress::ResolveFromString()
-
+#ifdef USE_GETHOSTBYADDR
 bool ProtoAddress::ResolveToName(char* buffer, unsigned int buflen) const
 {
 #ifdef WIN32
@@ -1423,6 +1462,54 @@ bool ProtoAddress::ResolveToName(char* buffer, unsigned int buflen) const
     }
 }  // end ProtoAddress::ResolveToName() 
 
+#else // Use newer getnameinfo rather than legacy gethostbyaddr
+bool ProtoAddress::ResolveToName(char* buffer, unsigned int buflen) const
+{
+#ifdef WIN32
+        if (!Win32Startup())
+        {
+            PLOG(PL_ERROR, "ProtoAddress::ResolveToName() Error initializing WinSock!\n");
+            GetHostString(buffer, buflen);
+            return false;
+        }
+        DWORD retval = 0;
+#else // WIN32
+        int retval = 0;
+#endif // endif WIN32
+    switch (type)
+    {
+        case IPv4:
+        case IPv6:
+            retval = getnameinfo((struct sockaddr *) &addr,
+                                 sizeof(addr),
+                                 buffer,
+                                 buflen,NULL,0,NI_NAMEREQD);
+            break;
+#ifdef SIMULATE
+        case SIM:
+            GetHostString(buffer, buflen);
+            return true;
+#endif // SIMULATE
+        case ETH:
+            GetHostString(buffer, buflen);
+            return true;
+        default:
+            PLOG(PL_ERROR, "ProtoAddress::ResolveToName(): Invalid address type!\n");
+            return false;
+    }  // end switch(type)
+#ifdef WIN32
+        Win32Cleanup();
+#endif // WIN32
+        if (retval != 0)
+        {
+            PLOG(PL_ERROR, "ProtoAddress::ResolveToName() error: %s\n", gai_strerror(retval));
+            return false;
+        }
+        
+        return true;
+}  // end ProtoAddress::ResolveToName() 
+#endif // endif USE_GETHOSTBYADDR
+
 bool ProtoAddress::ResolveLocalAddress(char* buffer, unsigned int buflen)
 {
     UINT16 thePort = GetPort();  // save port number
