@@ -383,7 +383,7 @@ bool ProtoBitmask::SetBits(UINT32 index, UINT32 count)
     {
         mask[maskIndex] |= 0x00ff >> bitIndex;
         count -= bitRemainder;
-        INT32 nbytes = count >> 3;  
+        UINT32 nbytes = count >> 3;  
         memset(&mask[++maskIndex], 0xff, nbytes);
         count &= 0x07;  
         if (count)
@@ -416,13 +416,13 @@ bool ProtoBitmask::UnsetBits(UINT32 index, UINT32 count)
     {
         mask[maskIndex] &= 0x00ff << bitRemainder;
         count -= bitRemainder;
-        INT32 nbytes = count >> 3;  
+        UINT32 nbytes = count >> 3;  
         memset(&mask[++maskIndex], 0, nbytes);
         count &= 0x07;  
         if (count) mask[maskIndex+nbytes] &= 0xff >> count;
     }
     if ((first_set >= index) && (end > first_set))
-    { 
+    {
         first_set = end;
         if (!GetNextSet(first_set)) first_set = num_bits;
     }
@@ -467,8 +467,14 @@ bool ProtoBitmask::Subtract(const ProtoBitmask& b)
 
 
 // this = ~this & b  (this = b - this)
+// (i.e., this is set to the bits uniquely set in 'b')
 bool ProtoBitmask::XCopy(const ProtoBitmask& b)
 {
+    if (!b.IsSet()) 
+    {
+        Clear();
+        return true;
+    }
     if (b.num_bits > num_bits) return false;
     unsigned int len = b.mask_len;
     unsigned int begin = b.first_set >> 3;
@@ -552,19 +558,16 @@ ProtoSlidingMask::~ProtoSlidingMask()
     Destroy();
 }
 
-bool ProtoSlidingMask::Init(INT32 numBits, UINT32 rangeMask)
+bool ProtoSlidingMask::Init(UINT32 numBits, UINT32 rangeMask)
 {
     if (mask) Destroy();
-    
-    //TRACE("ProtoSlidingMask::Init() rangeMask>%lu (0x%0x)\n", rangeMask, rangeMask);
-    
-    if ((numBits <= 0) || ((UINT32)numBits > ((rangeMask>>1)+1))) 
+    if ((0 != rangeMask) && (numBits > ((rangeMask>>1)+1))) 
         return false;
     UINT32 len = (numBits + 7) >> 3;
     if ((mask = new unsigned char[len]))
     {
         range_mask = rangeMask;
-        range_sign = (rangeMask ^ (rangeMask >> 1));
+        range_sign = rangeMask ? (rangeMask ^ (rangeMask >> 1)) : 0;
         mask_len = len;
         num_bits = numBits;
         Clear();
@@ -576,13 +579,17 @@ bool ProtoSlidingMask::Init(INT32 numBits, UINT32 rangeMask)
     }
 }  // end ProtoSlidingMask::Init()
 
-bool ProtoSlidingMask::Resize(INT32 numBits)
+bool ProtoSlidingMask::Resize(UINT32 numBits)
 {
     // 1) Backup the current state
     ProtoSlidingMask tempMask = *this;
     // 2) Prune if needed.
     if (numBits < num_bits)
-        tempMask.UnsetBits(offset+numBits, num_bits-numBits); 
+    {
+        UINT32 endex = offset + numBits;
+        if (range_mask) endex &= range_mask;
+        tempMask.UnsetBits(endex, num_bits-numBits); 
+    }
     // 3) Re-init w/ new "numBits"
     mask = NULL;
     bool result =  Init(numBits, range_mask);
@@ -603,7 +610,7 @@ bool ProtoSlidingMask::Resize(INT32 numBits)
 
 void ProtoSlidingMask::Destroy()
 {
-    if (mask)
+    if (NULL != mask)
     {
         delete[] mask;
         mask = NULL;
@@ -613,17 +620,17 @@ void ProtoSlidingMask::Destroy()
 
 bool ProtoSlidingMask::CanSet(UINT32 index) const
 {
+    ASSERT((0 == range_mask) || (index <= range_mask));
     if (IsSet())
     {
         // Determine position with respect to current start
         // and end, given the "offset" of the current start  
-        INT32 pos = Delta(index, offset);
-        if (pos < 0)
+        if (Compare(index, offset) < 0)
         {
             // Precedes start.
-            pos += start;
-            if (pos < 0) pos += num_bits;
-            if (pos < 0)
+            INT32 deltaPos = start + Difference(index, offset);;
+            if (deltaPos < 0) deltaPos += num_bits;
+            if (deltaPos < 0)
             {
                 // out of range
                 return false;
@@ -631,6 +638,7 @@ bool ProtoSlidingMask::CanSet(UINT32 index) const
             else
             {
                 // Is pos between end & start?
+                UINT32 pos = (UINT32)deltaPos;
                 if (end < start)
                 {
                    if ((pos <= end) || (pos >= start)) return false; 
@@ -642,7 +650,7 @@ bool ProtoSlidingMask::CanSet(UINT32 index) const
                 return true;
             } 
         }
-        else if (pos < num_bits)
+        else if ((UINT32)Difference(index, offset) < num_bits)
         {
                 return true;
         }
@@ -659,23 +667,25 @@ bool ProtoSlidingMask::CanSet(UINT32 index) const
 
 bool ProtoSlidingMask::Set(UINT32 index)
 {
+    ASSERT((0 == range_mask) || (index <= range_mask));
     if (IsSet())
     {        
         // Determine position with respect to current start
-        // and end, given the "offset" of the current start               
-        INT32 pos = Delta(index, offset);  
-        if (pos < 0)
+        // and end, given the "offset" of the current start   
+        UINT32 pos;            
+        if (Compare(index, offset) < 0)
         {
             // Precedes start.
-            pos += start;
-            if (pos < 0) pos += num_bits;
-            if (pos < 0)
+            INT32 deltaPos = start + Difference(index, offset);
+            if (deltaPos < 0) deltaPos += num_bits;
+            if (deltaPos < 0)
             {
                 // out of range
                 return false;
             }
             else
             {
+                pos = (UINT32)deltaPos;
                 // Is pos between end & start?
                 if (end < start)
                 {
@@ -690,25 +700,29 @@ bool ProtoSlidingMask::Set(UINT32 index)
                 offset = index;
             } 
         }
-        else if (pos < num_bits)
+        else 
         {
-            pos += start;
-            if (pos >= num_bits) pos -= num_bits;
-            if (end < start)
+            pos = Difference(index, offset);
+            if (pos < num_bits)
             {
-                if ((pos < start) && (pos > end)) end = pos;  
+                pos += start;
+                if (pos >= num_bits) pos -= num_bits;
+                if (end < start)
+                {
+                    if ((pos < start) && (pos > end)) end = pos;  
+                }
+                else
+                {
+                    if ((pos > end) || (pos < start)) end = pos;
+                }
             }
             else
             {
-                if ((pos > end) || (pos < start)) end = pos;
+                return false;  // out of range
             }
         }
-        else
-        {
-            return false;  // out of range
-        }
         ASSERT((pos >> 3) >= 0);
-        ASSERT((pos >> 3) < (INT32)mask_len);
+        ASSERT((pos >> 3) < mask_len);
         mask[(pos >> 3)] |= (0x80 >> (pos & 0x07));
     }
     else
@@ -722,99 +736,102 @@ bool ProtoSlidingMask::Set(UINT32 index)
 
 bool ProtoSlidingMask::Unset(UINT32 index)
 {
-    //ASSERT(CanSet(index));
+    ASSERT((0 == range_mask) || (index <= range_mask));
     if (IsSet())
     {
-        INT32 pos = Delta(index, offset);
-        if (pos < 0)
+        if (Compare(index, offset) < 0)
         {
             return true;  // out-of-range
         }
-        else if (pos < num_bits)
+        else 
         {
-            // Is it in current range of set bits?
-            pos += start;
-            if (pos >= num_bits) pos -= num_bits;
-            if (end < start)
+            UINT32 pos = Difference(index, offset);
+            if (pos < num_bits)
             {
-                if ((pos > end) && (pos < start)) return true;   
+                // Is it in current range of set bits?
+                pos += start;
+                if (pos >= num_bits) pos -= num_bits;
+                if (end < start)
+                {
+                    if ((pos > end) && (pos < start)) return true;   
+                }
+                else
+                {
+                    if ((pos < start) || (pos > end)) return true;   
+                }
+                // Yes, it was in range.
+                // Unset the corresponding bit
+                ASSERT((pos >> 3) >= 0);
+                ASSERT((pos >> 3) < mask_len);
+                mask[(pos >> 3)] &= ~(0x80 >> (pos & 0x07));
+                if (start == end) 
+                {
+                    ASSERT(pos == start);
+                    start = end = num_bits;
+                    return true;
+                }
+                if (start == pos) 
+                {
+                    UINT32 next = index;
+                    if (!GetNextSet(next)) 
+                        ASSERT(0);
+                    ASSERT(Compare(next, offset) >= 0);
+                    UINT32 delta = Difference(next, offset);
+                    start += delta;
+                    if (start >= num_bits) start -= num_bits;
+                    offset = next;
+                }
+                if (pos == end) 
+                {
+                    UINT32 prev = index;
+                    if (!GetPrevSet(prev)) 
+                        ASSERT(0);
+                    ASSERT(Compare(prev, offset) >= 0);
+                    UINT32 delta = Difference(prev, offset);
+                    end = start + delta;
+                    if (end >= num_bits) end -= num_bits;
+                }
             }
             else
             {
-                if ((pos < start) || (pos > end)) return true;   
+                return true; // out-of-range   
             }
-            // Yes, it was in range.
-            // Unset the corresponding bit
-            ASSERT((pos >> 3) >= 0);
-            ASSERT((pos >> 3) < (INT32)mask_len);
-            mask[(pos >> 3)] &= ~(0x80 >> (pos & 0x07));
-            if (start == end) 
-            {
-                ASSERT(pos == start);
-                start = end = num_bits;
-                return true;
-            }
-            if (start == pos) 
-            {
-                UINT32 next = index;
-                if (!GetNextSet(next)) 
-                    ASSERT(0);
-                INT32 delta = Delta(next, offset);
-                ASSERT(delta >= 0);
-                start += delta;
-                if (start >= num_bits) start -= num_bits;
-                offset = next;
-            }
-            if (pos == end) 
-            {
-                UINT32 prev = index;
-                if (!GetPrevSet(prev)) 
-                    ASSERT(0);
-                INT32 delta = Delta(prev, offset);
-                ASSERT(delta >= 0);
-                end = start + delta;
-                if (end >= num_bits) end -= num_bits;
-            }
-        }
-        else
-        {
-            return true; // out-of-range   
         }
     }
     return true;
 }  // end ProtoSlidingMask::Unset()
 
-bool ProtoSlidingMask::SetBits(UINT32 index, INT32 count)
+bool ProtoSlidingMask::SetBits(UINT32 index, UINT32 count)
 {
-    if (count < 0) return false;
+    ASSERT((0 == range_mask) || (index <= range_mask));
     if (0 == count) return true;
-    INT32 firstPos, lastPos;
+    UINT32 firstPos, lastPos;
     if (IsSet())
     {
-        INT32 last = (index + count - 1) & range_mask;
+        UINT32 last = (index + count - 1);
+        if (range_mask) last &= range_mask;
         if (!CanSet(index)) return false;
         if (!CanSet(last)) return false; 
         // Calculate first set bit position  
-        firstPos = Delta(index, offset);
-        if (firstPos < 0)
+        if (Compare(index, offset) < 0)
         {
             // precedes start
-            firstPos += start;
-            if (firstPos < 0) firstPos += num_bits;
-            start = firstPos;
+            INT32 deltaPos = start + Difference(index, offset);
+            if (deltaPos < 0) deltaPos += num_bits;
+            // The "CanSet()" checks above guarantee positive "deltaPos" here
+            start = firstPos = (UINT32)deltaPos;
             offset = index;
         }
         else
         {
-            firstPos += start;
+            firstPos = start + Difference(index, offset);
             if (firstPos >= num_bits) firstPos -= num_bits;
         }
         // Calculate last set bit position
-        lastPos = Delta(last , offset);
-        if (lastPos > 0)
+        if (Compare(last, offset) > 0)
         {
             // Is post start, post end?  
-            lastPos += start; 
+            lastPos = start + Difference(last, offset); 
             if (lastPos >= num_bits) lastPos -= num_bits;
             if (end < start)
             {
@@ -827,36 +844,35 @@ bool ProtoSlidingMask::SetBits(UINT32 index, INT32 count)
         }
         else
         {
-            lastPos += start;
-            if (lastPos < 0) lastPos += num_bits;
+            INT32 deltaPos = start + Difference(last, offset);
+            if (deltaPos < 0) deltaPos += num_bits;
+            // The "CanSet()" checks above guarantee positive "deltaPos" here
+            lastPos = (UINT32)deltaPos;
         }
         if (lastPos < firstPos)
         {
             // Set bits from firstPos to num_bits   
             count = num_bits - firstPos;
-            INT32 maskIndex = firstPos >> 3;
-            int bitIndex = firstPos & 0x07;
-            int bitRemainder = 8 - bitIndex;
-            ASSERT(maskIndex >= 0);
+            UINT32 maskIndex = firstPos >> 3;
+            UINT32 bitIndex = firstPos & 0x07;
+            UINT32 bitRemainder = 8 - bitIndex;
+            ASSERT(maskIndex < mask_len);
             if (count <= bitRemainder)
             {
-                ASSERT(maskIndex < (INT32)mask_len);
                 mask[maskIndex] |= (0x00ff >> bitIndex) &
                                    (0x00ff << (bitRemainder - count)); 
             }
             else
             {
-                ASSERT(maskIndex < (INT32)mask_len);
                 mask[maskIndex] |= 0x00ff >> bitIndex;
                 count -= bitRemainder;
-                INT32 nbytes = count >> 3;  
-                ASSERT((maskIndex+1+nbytes) <= (INT32)mask_len);
+                UINT32 nbytes = count >> 3;  
+                ASSERT((maskIndex+1+nbytes) <= mask_len);
                 memset(&mask[++maskIndex], 0xff, nbytes);
                 count &= 0x07;  
                 if (count) 
                 {
-                    ASSERT((maskIndex+nbytes) >= 0);
-                    ASSERT((maskIndex+nbytes) < (INT32)mask_len);
+                    ASSERT((maskIndex+nbytes) < mask_len);
                     mask[maskIndex+nbytes] |= 0xff << (8-count);
                 }
             }
@@ -867,98 +883,100 @@ bool ProtoSlidingMask::SetBits(UINT32 index, INT32 count)
     {
         if (count > num_bits) return false;
         start = firstPos = 0;
-        end = lastPos = (INT32)(count - 1);
+        end = lastPos = (count - 1);
         offset = index;
     }
     // Set bits from firstPos to lastPos   
     count = lastPos - firstPos + 1;
-    INT32 maskIndex = firstPos >> 3;
-    int bitIndex = firstPos & 0x07;
-    int bitRemainder = 8 - bitIndex;
-    ASSERT(maskIndex >= 0);
+    UINT32 maskIndex = firstPos >> 3;
+    UINT32 bitIndex = firstPos & 0x07;
+    UINT32 bitRemainder = 8 - bitIndex;
+    ASSERT(maskIndex < mask_len);
     if (count <= bitRemainder)
     {
-        ASSERT(maskIndex < (INT32)mask_len);
         mask[maskIndex] |= (0x00ff >> bitIndex) &
                             (0x00ff << (bitRemainder - count)); 
     }
     else
     {
-        ASSERT(maskIndex < (INT32)mask_len);
         mask[maskIndex] |= 0x00ff >> bitIndex;
         count -= bitRemainder;
-        INT32 nbytes = count >> 3;
-        ASSERT((maskIndex+1+nbytes) <= (INT32)mask_len);  
+        UINT32 nbytes = count >> 3;
+        ASSERT((maskIndex+1+nbytes) <= mask_len);  
         memset(&mask[++maskIndex], 0xff, nbytes);
         count &= 0x07;  
         if (count) 
         {
-            ASSERT((maskIndex+nbytes) >= 0);
-            ASSERT((maskIndex+nbytes) < (INT32)mask_len);
+            ASSERT((maskIndex+nbytes) < mask_len);
             mask[maskIndex+nbytes] |= 0xff << (8-count);
         }
     }
     return true;
 }  // end ProtoSlidingMask::SetBits()
 
-bool ProtoSlidingMask::UnsetBits(UINT32 index, INT32 count)
+bool ProtoSlidingMask::UnsetBits(UINT32 index, UINT32 count)
 {
+    ASSERT((0 == range_mask) || (index <= range_mask));
     if (IsSet())
     {
         // Trim to fit as needed.
-        INT32 firstPos;
-        if (count <= 0) return true;
+        UINT32 firstPos;
+        if (0 == count) return true;
         if (count > num_bits) count = num_bits;
-        INT32 delta = Delta(index, offset);
-        if (delta >= num_bits)
+        if (Compare(index, offset) < 0)
         {
-            return true;
-        }
-        else if (delta <= 0)
-        {
-            firstPos = start;  
-            count += delta; 
-            if (count <= 0) return true;
+            // index precedes offset
+            firstPos = start;
+            UINT32 diff = Difference(offset, index);
+            if (diff >= count)
+            {
+                count = 0;
+                return true;
+            }
+            count -= diff;
         }
         else
         {
-            firstPos = start + delta;
+            UINT32 diff = Difference(index, offset);
+            if (diff >= num_bits) 
+                return true; //beyond range
+            firstPos = start + diff;
             if (firstPos >= num_bits) firstPos -= num_bits;
-        }
+        } 
         UINT32 lastSet;
         if (!GetLastSet(lastSet)) 
             ASSERT(0);
-        delta = Delta(((index+count-1) & range_mask), lastSet);
-        INT32 lastPos;
-        if (delta < 0)
-        { 
+        UINT32 endex = index + count - 1;
+        if (range_mask) endex &= range_mask;
+        UINT32 lastPos;
+        if (Compare(endex, lastSet) < 0)
+        {
             lastPos = firstPos + count - 1;
-            if (lastPos >= num_bits) lastPos -= num_bits;
+            if (lastPos >= num_bits) lastPos -= num_bits;   
         }
         else
         {
             lastPos = end;
         }
-        INT32 startPos;
+        UINT32 startPos;
         if (lastPos < firstPos)
         {
             // Clear bits from firstPos to num_bits   
             count = num_bits - firstPos;
-            INT32 maskIndex = firstPos >> 3;
-            int bitIndex = firstPos & 0x07;
-            int bitRemainder = 8 - bitIndex;
+            UINT32 maskIndex = firstPos >> 3;
+            UINT32 bitIndex = firstPos & 0x07;
+            UINT32 bitRemainder = 8 - bitIndex;
+            ASSERT(maskIndex < mask_len);
             if (count <= bitRemainder)
             {
-                ASSERT(maskIndex < (INT32)mask_len);
                 mask[maskIndex] &= (0x00ff << bitRemainder) |
                                    (0x00ff >> (bitIndex + count));
             }
             else
             {
-                ASSERT(maskIndex < mask_len);
                 mask[maskIndex] &= 0x00ff << bitRemainder;
                 count -= bitRemainder;
-                INT32 nbytes = count >> 3;  
+                UINT32 nbytes = count >> 3;  
                 ASSERT((maskIndex+1+nbytes) <= mask_len);
                 memset(&mask[++maskIndex], 0, nbytes);
                 count &= 0x07;  
@@ -976,21 +994,20 @@ bool ProtoSlidingMask::UnsetBits(UINT32 index, INT32 count)
         }
         // Unset bits from firstPos to lastPos   
         count = lastPos - startPos + 1;
-        INT32 maskIndex = startPos >> 3;
-        int bitIndex = startPos & 0x07;
-        int bitRemainder = 8 - bitIndex;
+        UINT32 maskIndex = startPos >> 3;
+        UINT32 bitIndex = startPos & 0x07;
+        UINT32 bitRemainder = 8 - bitIndex;
+        ASSERT(maskIndex < mask_len);
         if (count <= bitRemainder)
         {
-            ASSERT(maskIndex < mask_len);
             mask[maskIndex] &= (0x00ff << bitRemainder) |
                                (0x00ff >> (bitIndex + count)); 
         }
         else
         {
-            ASSERT(maskIndex < mask_len);
             mask[maskIndex] &= 0x00ff << bitRemainder;
             count -= bitRemainder;
-            INT32 nbytes = count >> 3;
+            UINT32 nbytes = count >> 3;
             ASSERT((maskIndex+1+nbytes) <= mask_len);
             memset(&mask[++maskIndex], 0, nbytes);
             count &= 0x07;  
@@ -1024,11 +1041,12 @@ bool ProtoSlidingMask::UnsetBits(UINT32 index, INT32 count)
 
 bool ProtoSlidingMask::Test(UINT32 index) const
 {
+    ASSERT((0 == range_mask) || (index <= range_mask));
     if (IsSet())
     {
-        INT32 pos = Delta(index , offset);
-        if (pos >= 0)
+        if (Compare(index, offset) >= 0)
         {
+            UINT32 pos = Difference(index, offset);
             // Is it in range?
             if (pos >= num_bits) return false;
             pos += start;
@@ -1049,15 +1067,15 @@ bool ProtoSlidingMask::Test(UINT32 index) const
     return false;
 }  // end ProtoSlidingMask::Test()
 
-
 bool ProtoSlidingMask::GetNextSet(UINT32& index) const
 {
+    ASSERT((0 == range_mask) || (index <= range_mask));
     if (IsSet())
     {
         UINT32 next = index;
-        INT32 pos = Delta(next, offset);
-        if (pos >= 0)
+        if (Compare(next, offset) >= 0)
         {
+            UINT32 pos = Difference(next, offset);
             // Is it in range?
             if (pos >= num_bits) return false;
             pos += start;
@@ -1072,7 +1090,7 @@ bool ProtoSlidingMask::GetNextSet(UINT32& index) const
                 if ((pos < start) || (pos > end)) return false;
             }
             // Seek next set bit
-            INT32 maskIndex = pos >> 3;
+            UINT32 maskIndex = pos >> 3;
             if (mask[maskIndex])
             {
                 int w = ProtoBitmask::WEIGHT[mask[maskIndex]];
@@ -1083,10 +1101,12 @@ bool ProtoSlidingMask::GetNextSet(UINT32& index) const
                     if (loc >= remainder) 
                     {
                         pos = (maskIndex << 3) + loc;
-                        pos -= start;
-                        if (pos < 0) pos += num_bits;
+                        if (pos >= start)
+                            pos -= start;
+                        else
+                            pos = num_bits - (start - pos);
                         index = offset + pos;
-                        index &= range_mask;
+                        if (range_mask) index &= range_mask;
                         return true;
                     }
                 }
@@ -1099,25 +1119,29 @@ bool ProtoSlidingMask::GetNextSet(UINT32& index) const
                     if (mask[maskIndex])
                     {
                         pos = (maskIndex << 3) + ProtoBitmask::BITLOCS[mask[maskIndex]][0];
-                        pos -= start;
-                        if (pos < 0) pos += num_bits;
-                        index =  offset + pos;
-                        index &= range_mask;
+                        if (pos >= start)
+                            pos -= start;
+                        else
+                            pos = num_bits - (start - pos);
+                        index = offset + pos;
+                        if (range_mask) index &= range_mask;
                         return true;
                     }
                 }
                 maskIndex = 0;
             }
-            INT32 endIndex = end >> 3;
+            UINT32 endIndex = end >> 3;
             for (; maskIndex <= endIndex; maskIndex++)
             {
                 if (mask[maskIndex])
                 {
                     pos = (maskIndex << 3) + ProtoBitmask::BITLOCS[mask[maskIndex]][0];
-                    pos -= start;
-                    if (pos < 0) pos += num_bits;
+                    if (pos >= start)
+                        pos -= start;
+                    else
+                        pos = num_bits - (start - pos);
                     index = offset + pos;
-                    index &= range_mask;
+                    if (range_mask) index &= range_mask;
                     return true;
                 }
             }
@@ -1133,12 +1157,13 @@ bool ProtoSlidingMask::GetNextSet(UINT32& index) const
 
 bool ProtoSlidingMask::GetPrevSet(UINT32& index) const
 {
+    ASSERT((0 == range_mask) || (index <= range_mask));
     if (IsSet())
     {
         UINT32 prev = index;
-        INT32 pos = Delta(prev, offset);
-        if (pos >= 0)
+        if (Compare(prev, offset) >= 0)
         {
+            UINT32 pos = Difference(prev, offset);
             // Is it in range?
             if (pos >= num_bits) 
             {
@@ -1164,7 +1189,7 @@ bool ProtoSlidingMask::GetPrevSet(UINT32& index) const
                 }
             }
             // Seek prev set bits, starting with index   
-            INT32 maskIndex = pos >> 3;
+            UINT32 maskIndex = pos >> 3;
             if (mask[maskIndex])
             {
                 int w = ProtoBitmask::WEIGHT[mask[maskIndex]] - 1;
@@ -1175,10 +1200,12 @@ bool ProtoSlidingMask::GetPrevSet(UINT32& index) const
                     if (loc <= remainder) 
                     {
                         pos = (maskIndex << 3) + loc;
-                        pos -= start;
-                        if (pos < 0) pos += num_bits;
+                        if (pos >= start)
+                            pos -= start;
+                        else
+                            pos = num_bits - (start - pos);
                         index = offset + pos;
-                        index &= range_mask;
+                        if (range_mask) index &= range_mask;
                         return true;
                     }
                 }
@@ -1186,33 +1213,38 @@ bool ProtoSlidingMask::GetPrevSet(UINT32& index) const
             maskIndex--;
             if (pos < start) 
             {
-                for(; maskIndex >= 0; maskIndex--)
+                //for(; maskIndex >= 0; maskIndex--)
+                for(; maskIndex != 0xffffffff; maskIndex--)
                 {                   
                     if (mask[maskIndex])
                     {
                         
                         int w = ProtoBitmask::WEIGHT[mask[maskIndex]] - 1;
                         pos =  (maskIndex << 3) + ProtoBitmask::BITLOCS[mask[maskIndex]][w];
-                        pos -= start;
-                        if (pos < 0) pos += num_bits;
+                        if (pos >= start)
+                            pos -= start;
+                        else
+                            pos = num_bits - (start - pos);
                         index = offset + pos;
-                        index &= range_mask;
+                        if (range_mask) index &= range_mask;
                         return true;  
                     }
                 } 
                 maskIndex = mask_len - 1;  
             }
-            INT32 startIndex = start >> 3;
+            UINT32 startIndex = start >> 3;
             for (; maskIndex >= startIndex; maskIndex--)
             {
                 if (mask[maskIndex])
                 {
                     int w = ProtoBitmask::WEIGHT[mask[maskIndex]] - 1;
                     pos =  (maskIndex << 3) + ProtoBitmask::BITLOCS[mask[maskIndex]][w];
-                    pos -= start;
-                    if (pos < 0) pos += num_bits;
+                    if (pos >= start)
+                        pos -= start;
+                    else
+                        pos = num_bits - (start - pos);
                     index = offset + pos;
-                    index &= range_mask;
+                    if (range_mask) index &= range_mask;
                     return true;  
                 }
             }
@@ -1225,8 +1257,9 @@ bool ProtoSlidingMask::Copy(const ProtoSlidingMask& b)
 {
     if (b.IsSet())
     {
-        INT32 range = b.end - b.start;
-        if (range < 0) range += b.num_bits;
+        UINT32 range = (b.end >= b.start) ?
+                            (b.end - b.start) :
+                            (b.num_bits - (b.start - b.end));
         if (range <= num_bits)
         {
             start = b.start & 0x07;
@@ -1234,11 +1267,11 @@ bool ProtoSlidingMask::Copy(const ProtoSlidingMask& b)
             b.GetLastSet(bLastSet);
             UINT32 bFirstSet;
             b.GetFirstSet(bFirstSet);
-            end = bLastSet - bFirstSet + start;
+            end = Difference(bLastSet,  bFirstSet) + start;
             offset = b.offset;
             // Copy start to mask_len
-            INT32 startIndex = b.start >> 3;
-            INT32 endIndex = b.end >> 3;
+            UINT32 startIndex = b.start >> 3;
+            UINT32 endIndex = b.end >> 3;
             if (b.end < b.start)
             {
                 ASSERT((b.mask_len - startIndex) <= mask_len);
@@ -1291,16 +1324,19 @@ bool ProtoSlidingMask::Add(const ProtoSlidingMask& b)
             UINT32 bLastSet;
             b.GetFirstSet(bLastSet);
             if (!CanSet(bLastSet)) return false;
-            INT32 range = b.end - b.start;
-            if (range < 0) range += b.num_bits;
+            
+            UINT32 range = (b.end >= b.start) ?
+                                (b.end - b.start) :
+                                (b.num_bits - (b.start - b.end));
             UINT32 index;
             b.GetFirstSet(index);
-            for (INT32 i = 0; i < range; i++)
+            for (UINT32 i = 0; i <= range; i++)
             {
                 // (TBD) Improve performance by getting/setting
                 //       ranges of set bits.
                 if (b.Test(index)) Set(index);
                 index++; 
+                if (range_mask) index &= range_mask;
             }
             return true;
         }
@@ -1316,18 +1352,17 @@ bool ProtoSlidingMask::Add(const ProtoSlidingMask& b)
 // This leaves us with bits uniquely set in "this" 
 bool ProtoSlidingMask::Subtract(const ProtoSlidingMask& b)
 {
-    if (b.IsSet())
+    if (IsSet() && b.IsSet())
     {
-        if (IsSet())
+        UINT32 index = offset;
+        UINT32 range = (end >= start) ?
+                            (end - start) :
+                            (num_bits - (start - end));
+        for (UINT32 i = 0; i <= range; i++)
         {
-            UINT32 index = offset;
-            INT32 range = end - start;
-            if (range < 0) range += num_bits;
-            for (INT32 i = 0; i < range; i++)
-            {
-                if (Test(index) && b.Test(index)) Unset(index);
-                index++;   
-            }
+            if (Test(index) && b.Test(index)) Unset(index);
+            index++;   
+            if (range_mask) index &= range_mask;
         } 
     }
     return true;
@@ -1341,23 +1376,42 @@ bool ProtoSlidingMask::XCopy(const ProtoSlidingMask& b)
     {
         if (IsSet())
         {
+            // Make sure b's range is compatible
             UINT32 bFirstSet;
             b.GetFirstSet(bFirstSet);
             if (!CanSet(bFirstSet)) return false;
             UINT32 bLastSet;
             b.GetFirstSet(bLastSet);
             if (!CanSet(bLastSet)) return false;
-            UINT32 index;
-            b.GetFirstSet(index);
-            INT32 range = b.end - b.start;
-            if (range < 0) range += b.num_bits;
-            for (INT32 i = 0; i < range; i++)
+            UINT32 firstSet;
+            GetFirstSet(firstSet);
+            UINT32 lastSet;
+            GetLastSet(lastSet);
+            // Clear any bits prior to 'bFirstSet'
+            if (Compare(firstSet, bFirstSet) < 0)
+            {
+                UINT32 numBits = Difference(bFirstSet, firstSet);
+                UnsetBits(firstSet, numBits);
+            }
+            // Clear any bits after 'bLastSet'
+            if (Compare(lastSet, bLastSet) > 0)
+            {
+                UINT32 numBits = Difference(lastSet, bLastSet);
+                UnsetBits(bLastSet, numBits);
+            }
+            // Perform XCopy() operation on the common range
+            UINT32 range = (b.end >= b.start) ?
+                                (b.end - b.start) : 
+                                (b.num_bits - (b.start - b.end));
+            UINT32 index = bFirstSet;
+            for (UINT32 i = 0; i <= range; i++)
             {
                 if (Test(index))
                     Unset(index);
                 else if (b.Test(index))
                     Set(index);
-                index++;   
+                index++;
+                if (b.range_mask) index &= range_mask;
             }
         }
         else
@@ -1380,12 +1434,14 @@ bool ProtoSlidingMask::Multiply(const ProtoSlidingMask& b)
         if (IsSet())
         {
             UINT32 index = offset;
-            INT32 range = end - start;
-            if (range < 0) range += num_bits;
-            for (INT32 i = 0; i < range; i++)
+            UINT32 range = (end >= start) ?
+                                (end - start) :
+                                (num_bits - (start - end));
+            for (UINT32 i = 0; i <= range; i++)
             {
                 if (Test(index) && !b.Test(index)) Unset(index);
-                index++;   
+                index++;     
+                if (range_mask) index &= range_mask; 
             }
         } 
     }
@@ -1410,12 +1466,14 @@ bool ProtoSlidingMask::Xor(const ProtoSlidingMask& b)
         if (!CanSet(bLastSet)) return false;
         UINT32 index;
         b.GetFirstSet(index);
-        INT32 range = b.end - b.start;
-        if (range < 0) range += b.num_bits;
-        for (INT32 i = 0; i < range; i++)
+        UINT32 range = (b.end >= b.start) ?
+                            (b.end - b.start) :
+                            (b.num_bits - (b.start - b.end));
+        for (UINT32 i = 0; i <= range; i++)
         {
             if (b.Test(index)) Invert(index);
             index++;   
+            if (range_mask) index &= range_mask;   
         }
     }
     return true;
@@ -1424,25 +1482,29 @@ bool ProtoSlidingMask::Xor(const ProtoSlidingMask& b)
 void ProtoSlidingMask::Display(FILE* stream)
 {
     UINT32 index = offset;
-    for (INT32 i = 0; i < num_bits; i++)
+    for (UINT32 i = 0; i < num_bits; i++)
     {
-        if (Test(index++)) fprintf(stream, "1"); else fprintf(stream, "0");
+        if (Test(index)) fprintf(stream, "1"); else fprintf(stream, "0");
         if (0x07 == (i & 0x07)) fprintf(stream, " ");
         if (0x3f == (i & 0x3f)) fprintf(stream, "\n");
+        index++;   
+        if (range_mask) index &= range_mask;  
     }
 }  // end ProtoSlidingMask::Display()
 
-void ProtoSlidingMask::Debug(INT32 theCount)
+void ProtoSlidingMask::Debug(UINT32 theCount)
 {
     UINT32 index = offset;
     theCount = MIN(theCount, num_bits);
-    PLOG(PL_ERROR, "ProtoSlidingMask::Debug() offset:%lu\n   ", index);
-    INT32 i;
+    PLOG(PL_ERROR, "ProtoSlidingMask::Debug() offset:%lu\n   ", (unsigned long)index);
+    UINT32 i;
     for (i = 0; i < theCount; i++)
     {
-        if (Test(index++)) PLOG(PL_ERROR, "1"); else PLOG(PL_ERROR, "0");
+        if (Test(index)) PLOG(PL_ERROR, "1"); else PLOG(PL_ERROR, "0");
         if (0x07 == (i & 0x07)) PLOG(PL_ERROR, " ");
         if (0x3f == (i & 0x3f)) PLOG(PL_ERROR, "\n   ");
+        index++;   
+        if (range_mask) index &= range_mask;  
     }
     if (0x3f != (i & 0x3f)) PLOG(PL_ERROR, "\n");
 }  // end ProtoSlidingMask::Debug()
