@@ -152,24 +152,25 @@ bool ProtoPktIPv4::InitFromBuffer(UINT32* bufferPtr, unsigned int numBytes, bool
         ProtoPkt::SetLength(0);
     if (GetBufferLength() > (OFFSET_VERSION+1))
     {
-        if (4 != (((UINT8*)buffer_ptr)[0] >> 4))
+        if (4 == (((UINT8*)buffer_ptr)[0] >> 4))
         {
-            return false;  // not an IPv4 packet!
-        }
-        else if (GetBufferLength() > (OFFSET_LEN + 2))
-        {
-            // this validates that the embedded length is <= buffer_bytes
-            return ProtoPkt::InitFromBuffer(GetTotalLength());
+            if (GetBufferLength() > (OFFSET_LEN + 2) &&
+                ProtoPkt::InitFromBuffer(GetTotalLength()))
+            {
+                return true;
+            }
+            else
+            {
+                PLOG(PL_ERROR, "ProtoPktIPv4::InitFromBuffer() error: insufficient buffer space!\n");
+            }
         }
         else
         {
-            return false;
+            PLOG(PL_WARN, "ProtoPktIPv4::InitFromBuffer() error: invalid protocol version!\n");
         }
     }
-    else
-    {
-        return false;
-    }
+    if (NULL != bufferPtr) DetachBuffer();
+    return false;
 }  // end ProtoPktIPv4::InitFromBuffer()
 
 
@@ -190,6 +191,10 @@ bool ProtoPktIPv4::InitIntoBuffer(UINT32* bufferPtr, unsigned int bufferBytes, b
     SetHeaderLength(20);
     SetChecksum(0);
     ((UINT16*)buffer_ptr)[OFFSET_FRAGMENT] = 0;  // init flags & frag to ZERO
+    ((UINT8*)buffer_ptr)[OFFSET_TOS] = 0;
+    ((UINT8*)buffer_ptr)[OFFSET_FLAGS] = 0;
+    ((UINT8*)buffer_ptr)[OFFSET_TTL] = 255;
+    ((UINT8*)buffer_ptr)[OFFSET_PROTOCOL] = 0;
     // (TBD) Set some header fields to default values? (e.g. fragment, flags)
     // (TBD) should we set total length to 20 here? 
     return true;
@@ -579,7 +584,6 @@ bool ProtoPktIPv6::InitFromBuffer(UINT32* bufferPtr, unsigned int numBytes, bool
         if (6 != (((UINT8*)buffer_ptr)[0] >> 4))
         {
             PLOG(PL_ERROR, "ProtoPktIPv6::InitFromBuffer() error: invalid version number\n");
-            return false;  // not an IPv6 packet!
         }
         else if (GetBufferLength() > (2*OFFSET_LENGTH + 2))
         {
@@ -591,20 +595,19 @@ bool ProtoPktIPv6::InitFromBuffer(UINT32* bufferPtr, unsigned int numBytes, bool
             else
             {
                 PLOG(PL_ERROR, "ProtoPktIPv6::InitFromBuffer() error: invalid packet length?\n");
-                return false;
             }
         }
         else
         {
             PLOG(PL_ERROR, "ProtoPktIPv6::InitFromBuffer() error: insufficient buffer space (2)\n");
-            return false;
         }
     }
     else
     {
         PLOG(PL_ERROR, "ProtoPktIPv6::InitFromBuffer() error: insufficient buffer space (1)\n");
-        return false;
     }
+    if (NULL != bufferPtr) DetachBuffer();
+    return false;
 }  // end ProtoPktIPv6::InitFromBuffer()
 
 
