@@ -396,7 +396,7 @@ void NetGraph::Interface::PriorityQueue::Adjust(Interface& iface, const Cost& ne
     InsertItem(*item);
 }  // end ProtoGraph::Vertice::PriorityQueue::Adjust()
 
-bool NetGraph::Interface::PriorityQueue::AdjustDownward(Interface& iface, const Cost& newCost)
+bool NetGraph::Interface::PriorityQueue::AdjustDownward(Interface& iface, const Cost& newCost, const Interface* newPrevHop)
 {
     Item* item = static_cast<Item*>(GetQueueState(iface));
     ASSERT(item != NULL);
@@ -406,7 +406,7 @@ bool NetGraph::Interface::PriorityQueue::AdjustDownward(Interface& iface, const
         item->SetCost(newCost);
         InsertItem(*item);
         return true;
-    }
+    } 
     else
     {
         return false;
@@ -1127,7 +1127,7 @@ NetGraph::Interface* NetGraph::DijkstraTraversal::GetNextInterface()
                     {
                        // We have found another path to this pending iface.
                         //  If it is a shorter path update the cost to lower value
-                        if (!queue_pending.AdjustDownward(*nextDst, nextCost))
+                        if (!queue_pending.AdjustDownward(*nextDst, nextCost, currentIface))
                                 saveState = false;
                     }
                     else if (!nextDst->IsInQueue(queue_visited))
@@ -1170,7 +1170,7 @@ NetGraph::Interface* NetGraph::DijkstraTraversal::GetNextInterface()
                         }
                         
                     } 
-                    else if (queuePtr->AdjustDownward(*nextDst, nextCost)) //try and move it down
+                    else if (queuePtr->AdjustDownward(*nextDst, nextCost, currentIface)) //try and move it down
                     {
                         //TRACE(" %s moved downward\n",nextDst->GetAddress().GetHostString());
                         if(isInVisited)
