@@ -91,8 +91,7 @@ bool ManetGraphMLParser::AttributeKey::Init(const char* theIndex,const char* the
 }
 
 
-bool
-ManetGraphMLParser::AttributeKey::Set(const char* theIndex,const char* theName,const char* theType,const char* theDomain, const char* theOldIndex,const char*theDefault)
+bool ManetGraphMLParser::AttributeKey::Set(const char* theIndex,const char* theName,const char* theType,const char* theDomain, const char* theOldIndex,const char*theDefault)
 {
     if(NULL != index)
         delete[] index;
@@ -144,8 +143,8 @@ ManetGraphMLParser::AttributeKey::Set(const char* theIndex,const char* theName,c
     }
     return true;
 }
-bool
-ManetGraphMLParser::AttributeKey::SetType(const char* theType)
+
+bool ManetGraphMLParser::AttributeKey::SetType(const char* theType)
 {
     if((!strcmp(theType,"bool")) || 
        (!strcmp(theType,"Bool")) ||
@@ -318,6 +317,43 @@ ManetGraphMLParser::SetAttributeKey(const char* theName,const char* theType, con
 
     return false; //shouldn't get here
 }
+
+bool 
+ManetGraphMLParser::SetAttribute(const char* theName, const char* theValue)
+{
+    PLOG(PL_DETAIL,"ManetGraphMLParser::SetAttribute(name=%s,theValue=%s)\n",theName,theValue);
+    const char* theIndex = FindAttributeIndex(theName);
+    if(NULL == theIndex)
+    {
+        PLOG(PL_ERROR,"ManetGraphMLParser::SetAttribute(Graph): Error finding index for attribute %s\n",theName);
+        return false;
+    }
+    PLOG(PL_DETAIL,"ManetGraphMLParser::SetAttribute(Node,name=%s,theValue=%s) found index %s\n",theName,theValue,theIndex);
+    char theLookup[250];
+    GetLookup(theLookup,250);
+    ManetGraphMLParser::Attribute* theAttribute = attributelist.FindAttribute(theLookup,theIndex);
+//this doesn't work it only finds one not all!
+    //ManetGraphMLParser::Attribute* theAttribute = attributelist.Find(theLookup,strlen(theLookup)*8);
+    if(NULL == theAttribute)
+    {
+        PLOG(PL_DETAIL,"ManetGraphMLParser::SetAttribute(name=%s,theValue=%s) making new one\n",theName,theValue);
+        return AddAttribute(theName,theValue);
+    } 
+    else
+    {
+        char tempIndex[20];
+        strcpy(tempIndex,theIndex); //we need to copy it over because setting it will delete the old value.
+        PLOG(PL_DETAIL,"ManetGraphMLParser::SetAttribute(name=%s,theValue=%s) Updating old one with index=%s\n",theName,theValue,theIndex);
+        if(!theAttribute->Set(theLookup,tempIndex,theValue))
+        {
+            PLOG(PL_ERROR,"ManetGraphMLParser::SetAttribute(): Error setting the attribute\n");
+            return false;
+        }
+        return true;
+    }
+    return false; //should never get here
+}
+
 bool 
 ManetGraphMLParser::SetAttribute(NetGraph::Node& node, const char* theName, const char* theValue)
 {
@@ -457,6 +493,38 @@ ManetGraphMLParser::AddAttributeKey(const char* theName,const char* theType, con
     PLOG(PL_DETAIL,"ManetGraphMLParser::AddAttributeKey() return true\n");
     return true;
 }
+
+bool ManetGraphMLParser::AddAttribute(const char* theName, const char* theValue)
+{
+    const char* theIndex = FindAttributeIndex(theName);
+    if(NULL == theIndex)
+    {
+        PLOG(PL_ERROR,"ManetGraphMLParser::AddAttribute(): Error finding index for attribute %s\n",theName);
+        return false;
+    }
+    Attribute* newAttribute = new Attribute();
+    if(NULL == newAttribute)
+    {
+        PLOG(PL_ERROR,"ManetGraphMLParser::AddAttribute(): Error allocating attribute\n");
+        return false;
+    }
+     
+    char theLookup[250];    //TBD
+    GetLookup(theLookup,250);
+    if(!newAttribute->Init(theLookup,theIndex,theValue))
+    {
+        PLOG(PL_ERROR,"ManetGraphMLParser::AddAttribute(Node): Error init the attribute\n");
+        return false;
+    }
+    if(!attributelist.Insert(*newAttribute))
+    {
+        PLOG(PL_ERROR,"ManetGraphMLParser::AddAttribute(): Error inserting the attribute\n");
+        return false;
+    }
+    DMSG(7,"ManetGraphMLParser::AddAttribute(%s,%s) added successfully\n",theName,theValue); 
+    return true;
+}
+
 bool ManetGraphMLParser::AddAttribute(NetGraph::Node& node, const char* theName, const char* theValue)
 {
     const char* theIndex = FindAttributeIndex(theName);
@@ -545,11 +613,18 @@ bool ManetGraphMLParser::AddAttribute(NetGraph::Interface& interface, const char
     }
     return true;
 }
+
+bool ManetGraphMLParser::GetLookup(char* theLookup,unsigned int maxlen)
+{
+    sprintf(theLookup,"thisGraph");//don't name interfaces/links/nodes thisGraph!
+    return true;
+}
+
 bool ManetGraphMLParser::GetLookup(char* theLookup,unsigned int maxlen,NetGraph::Node& node)
 {
     if(strlen(GetString(node))>maxlen)
     {
-        PLOG(PL_ERROR,"ManetGraphMLParser::GetLookup(node) ndoe string is longer than max leng\n");
+        PLOG(PL_ERROR,"ManetGraphMLParser::GetLookup(node) node string is longer than max leng\n");
         return false;
     }
     sprintf(theLookup,"node:%s",GetString(node));
@@ -1061,7 +1136,7 @@ bool ManetGraphMLParser::Read(const char* path, NetGraph& graph)
 }  // end ManetGraphMLParser::Read()
 
 
-bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
+bool ManetGraphMLParser::Write(NetGraph& graph, const char* path, char* buffer, unsigned int* len_ptr)
 {
     PLOG(PL_INFO,"ManetGraphMLParser::Write: Enter!\n");
 /* Create a new XmlWriter for DOM, with no compression. */
@@ -1106,13 +1181,26 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
         PLOG(PL_ERROR,"ManetGraphMLParser::Write::testXmlWriterDoc: Error at writing key elements in header\n");
         return false;
     }
+    if(!WriteLocalAttributes(writerPtr))
+    {
+        PLOG(PL_ERROR,"ManetGraphMLParser::Write::testXmlWriterDoc: Error at writing graph attributes in header\n");
+        return false;
+    }
     
     /* We are done with the header so now we go through the actual graph and add each node and edge */
     /* We are adding each node */
     returnvalue = xmlTextWriterStartElement(writerPtr, BAD_CAST "graph");
-    if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error starting XML graph element\n"); return false;}
+    if (returnvalue < 0) 
+    {
+        PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error starting XML graph element\n"); 
+        return false;
+    }
     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "id",BAD_CAST XMLName);
-    if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error setting XML graph attribute id\n"); return false;}
+    if (returnvalue < 0) 
+    {
+        PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error setting XML graph attribute id\n"); 
+        return false;
+    }
     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "edgedefault",BAD_CAST "directed");
     if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error setting graph attribute directed\n"); return false;}
 
@@ -1123,16 +1211,22 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
     {
         //check to see if this is a default "node" interface
         //if(!iface->IsPort())
-        if(iface == iface->GetNode().GetDefaultInterface())
+        if (iface == iface->GetNode().GetDefaultInterface())
         {
             //Node& node = static_cast<Node&>(iface->GetNode());
             returnvalue = xmlTextWriterStartElement(writerPtr, BAD_CAST "node");
-            if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error adding XML node\n"); return false;}
+            if (returnvalue < 0) 
+            {
+               PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error adding XML node\n"); 
+               return false;
+            }
             if(iface->GetAddress().IsValid())
             {
                 returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "id", BAD_CAST iface->GetAddress().GetHostString());
                 //printf("writing node %s\n",iface->GetAddress().GetHostString());
-            } else {
+            }
+            else 
+            {
                 returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "id", BAD_CAST iface->GetName());
                 //printf("writing node %s\n",iface->GetName());
             }
@@ -1168,16 +1262,26 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
 //                if(portIface->IsPort())
                 {
                     returnvalue = xmlTextWriterStartElement(writerPtr, BAD_CAST "port");
-                    if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error adding XML node\n"); return false;}
+                    if (returnvalue < 0) 
+                    {
+                        PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error adding XML node\n"); 
+                        return false;
+                    }
                     if(portIface->GetAddress().IsValid())
                     {
                         returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "name", BAD_CAST portIface->GetAddress().GetHostString());
                         //printf("writing interface %s\n",iface->GetAddress().GetHostString());
-                    } else {
+                    } 
+                    else 
+                    {
                         returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "name", BAD_CAST portIface->GetName());
                         //printf("writing node %s\n",iface->GetName());
                     } 
-                    if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error adding setting node id\n"); return false;}
+                    if (returnvalue < 0) 
+                    {
+                         PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error adding setting node id\n"); 
+                         return false;
+                    }
                     //update attributes to the port/interface using the virutal function
                     if(!UpdateInterfaceAttributes(*portIface))
                     {
@@ -1197,12 +1301,20 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
                         return false;
                     }*/
                     returnvalue = xmlTextWriterEndElement(writerPtr);
-                    if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error ending node element\n"); return false;}
+                    if (returnvalue < 0) 
+                    {
+                        PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error ending node element\n"); 
+                        return false;
+                    }
                 }
             }
             //close up the node node element
             returnvalue = xmlTextWriterEndElement(writerPtr);
-            if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error ending node element\n"); return false;}
+            if (returnvalue < 0) 
+            {
+                PLOG(PL_ERROR,"ManetGraphMLParser::Write: Error ending node element\n"); 
+                return false;
+            }
         }
     }
     //adding the directional links to the xml
@@ -1217,23 +1329,38 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
             while (NULL != (nbrIface = iteratorN1.GetNextAdjacency()))
             {
                 returnvalue = xmlTextWriterStartElement(writerPtr, BAD_CAST "edge");
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding edge\n"); return false;}
-                if(iface->GetAddress().IsValid()) {
+                if (returnvalue < 0) 
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding edge\n"); return false;
+                }
+                if(iface->GetAddress().IsValid()) 
+                {
                     //printf("writing connection %s ->",iface->GetAddress().GetHostString());
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "source", BAD_CAST iface->GetAddress().GetHostString());
-                } else { 
+                }else 
+                { 
                     //printf("writing connection %s ->",iface->GetName());
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "source", BAD_CAST iface->GetName());
                 }
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); return false;}
+                if (returnvalue < 0) 
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); 
+                    return false;
+                }
                 if(nbrIface->GetAddress().IsValid()) {
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "target", BAD_CAST nbrIface->GetAddress().GetHostString());
                     ////printf("%s\n",nbrIface->GetAddress().GetHostString());
-                } else {
+                } 
+                else 
+                {
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "target", BAD_CAST nbrIface->GetName());
                     ////printf("%s\n",nbrIface->GetName());
                 }
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); return false;}
+                if (returnvalue < 0) 
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); 
+                    return false;
+                }
                 NetGraph::Link* link = iface->GetLinkTo(*nbrIface);
                 if(NULL == link)
                 {
@@ -1253,7 +1380,11 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
                     return false;
                 }
                 returnvalue = xmlTextWriterEndElement(writerPtr);
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error ending node element\n"); return false;}
+                if (returnvalue < 0) 
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error ending node element\n"); 
+                    return false;
+                }
             } 
         }
         else //it is a port interface so we need to find the "node" interface
@@ -1280,32 +1411,63 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
                     return false;
                 }
                 returnvalue = xmlTextWriterStartElement(writerPtr, BAD_CAST "edge");
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding edge\n"); return false;}
-
-                if(nodeIface->GetAddress().IsValid()) {
+                if (returnvalue < 0) 
+                { 
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding edge\n"); 
+                    return false;
+                }
+                if(nodeIface->GetAddress().IsValid()) 
+                {
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "source", BAD_CAST nodeIface->GetAddress().GetHostString());
-                } else { 
+                } 
+                else 
+                { 
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "source", BAD_CAST nodeIface->GetName());
                 }
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); return false;}
-                if(nbrNodeIface->GetAddress().IsValid()) {
+                if (returnvalue < 0) 
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); 
+                    return false;
+                }
+                if(nbrNodeIface->GetAddress().IsValid()) 
+                {
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "target", BAD_CAST nbrNodeIface->GetAddress().GetHostString());
-                } else {
+                } 
+                else 
+                {
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "target", BAD_CAST nbrNodeIface->GetName());
                 }
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); return false;}
-                if(iface->GetAddress().IsValid()) {
+                if (returnvalue < 0) 
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); 
+                    return false;
+                }
+                if(iface->GetAddress().IsValid()) 
+                {
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "sourceport", BAD_CAST iface->GetAddress().GetHostString());
-                } else { 
+                } 
+                else 
+                { 
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "sourceport", BAD_CAST iface->GetName());
                 }
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); return false;}
-                if(nbrIface->GetAddress().IsValid()) {
+                if (returnvalue < 0)
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); 
+                    return false;
+                }
+                if(nbrIface->GetAddress().IsValid())
+                {
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "targetport", BAD_CAST nbrIface->GetAddress().GetHostString());
-                } else {
+                }
+                else 
+                {
                     returnvalue = xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "targetport", BAD_CAST nbrIface->GetName());
                 }
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); return false;}
+                if (returnvalue < 0) 
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error adding setting source attribute\n"); 
+                    return false;
+                }
 
                 NetGraph::Link* link = iface->GetLinkTo(*nbrIface);
                 if(NULL == link)
@@ -1326,7 +1488,11 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
                     return false;
                 }
                 returnvalue = xmlTextWriterEndElement(writerPtr);
-                if (returnvalue < 0) { PLOG(PL_ERROR,"ManetGraphMLParser::Write Error ending node element\n"); return false;}
+                if (returnvalue < 0) 
+                {
+                    PLOG(PL_ERROR,"ManetGraphMLParser::Write Error ending node element\n"); 
+                    return false;
+                }
             } 
         }
     }
@@ -1336,8 +1502,30 @@ bool ManetGraphMLParser::Write(NetGraph& graph, const char* path)
 
     xmlFreeTextWriter(writerPtr);
 
-    xmlSaveFormatFileEnc(path, docPtr, MY_GRAPHML_ENCODING,1);
-
+    if(NULL != path)
+    {
+        xmlSaveFormatFileEnc(path, docPtr, MY_GRAPHML_ENCODING,1);
+    }
+    if(NULL != buffer && NULL != len_ptr)
+    {
+        //xmlOutputBuffer xmlbuff;
+        //xmlSaveFormatFileTo(&xmlbuff,docPtr,MY_GRAPHML_ENCODING,1);
+        //if (xmlbuff.written > (int)*len_ptr){
+        //    DMSG(0,"bunny in buffer section\n");
+        
+        xmlChar* tempout;
+        int size;
+        xmlDocDumpFormatMemoryEnc(docPtr,&tempout,&size,MY_GRAPHML_ENCODING,1);
+        if(size > (int)*len_ptr)
+        {
+            PLOG(PL_ERROR,"ManetGraphMLParser::Write: size of document %d is larger than the allocated space of %d\n",size, (int)*len_ptr);
+            return false;
+        }
+        *len_ptr = size;
+        memcpy(buffer,tempout,size);
+        xmlFree(tempout);
+        xmlCleanupParser();
+    }
     xmlFreeDoc(docPtr);
  
     return true;
@@ -1355,7 +1543,9 @@ ManetGraphMLParser::AttributeList::FindAttribute(const char *theLookup,const cha
         {
             attr = NULL;
             //we didn't find the entry
-        } else {
+        } 
+        else 
+        {
             if(strcmp(attr->GetIndex(),theIndex))
             {
                 //this attribute does not have the index we are looking for
@@ -1367,8 +1557,8 @@ ManetGraphMLParser::AttributeList::FindAttribute(const char *theLookup,const cha
     }
     return attr;
 }
-bool
-ManetGraphMLParser::WriteLocalKeys(xmlTextWriter* writerPtr)
+
+bool ManetGraphMLParser::WriteLocalKeys(xmlTextWriter* writerPtr)
 {
     int rv = 0;
     IndexKeylist::Iterator it(indexkeylist);
@@ -1429,8 +1619,9 @@ ManetGraphMLParser::WriteLocalKeys(xmlTextWriter* writerPtr)
         return false;
     return true;
 }
-bool 
-ManetGraphMLParser::WriteLocalNodeAttributes(xmlTextWriter* writerPtr,NetGraph::Node& theNode)
+
+
+bool  ManetGraphMLParser::WriteLocalNodeAttributes(xmlTextWriter* writerPtr,NetGraph::Node& theNode)
 {
     PLOG(PL_DETAIL,"ManetGraphMLParser::WriteLocalNodeAttributes: Enter\n");
     bool rv = true;
@@ -1447,7 +1638,9 @@ ManetGraphMLParser::WriteLocalNodeAttributes(xmlTextWriter* writerPtr,NetGraph::
             //PLOG(PL_DETAIL,"ManetGraphMLParser::WriteLocalNodeAttributes():mykey=\"%s\",lookup=\"%s\",key=\"%s\",value=\"%s\"\n",key,attr->GetLookup(),attr->GetIndex(),attr->GetValue());
             attr = NULL;
             //attr = it.GetNextItem();
-        } else {
+        } 
+        else 
+        {
             rv += xmlTextWriterStartElement(writerPtr, BAD_CAST "data");
             rv += xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "key",BAD_CAST attr->GetIndex());
             rv += xmlTextWriterWriteString(writerPtr, BAD_CAST attr->GetValue());
@@ -1458,8 +1651,42 @@ ManetGraphMLParser::WriteLocalNodeAttributes(xmlTextWriter* writerPtr,NetGraph::
     }
     return rv;
 }
-bool 
-ManetGraphMLParser::WriteLocalInterfaceAttributes(xmlTextWriter* writerPtr,NetGraph::Interface& theInterface)
+
+bool  ManetGraphMLParser::WriteLocalAttributes(xmlTextWriter* writerPtr)
+{
+    PLOG(PL_DETAIL,"ManetGraphMLParser::WriteLocalAttributes: Enter\n");
+    bool rv = true;
+    char key[255];//this should be dynamic or checks added TBD
+    sprintf(key,"thisGraph");
+    AttributeList::Iterator it(attributelist,false,key,strlen(key)*8);
+    //AttributeList::Iterator it(attributelist);
+    
+    Attribute* attr(NULL);
+    
+    //iterate over items which have the matching keys
+    attr = it.GetNextItem();
+    while(NULL != attr)
+    {
+        if(strcmp(attr->GetLookup(),key))
+        {
+            //PLOG(PL_DETAIL,"ManetGraphMLParser::WriteLocalAttributes():mykey=\"%s\",lookup=\"%s\",key=\"%s\",value=\"%s\"\n",key,attr->GetLookup(),attr->GetIndex(),attr->GetValue());
+            attr = NULL;
+            //attr = it.GetNextItem();
+        } 
+        else 
+        {
+            rv += xmlTextWriterStartElement(writerPtr, BAD_CAST "data");
+            rv += xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "key",BAD_CAST attr->GetIndex());
+            rv += xmlTextWriterWriteString(writerPtr, BAD_CAST attr->GetValue());
+            rv += xmlTextWriterEndElement(writerPtr);
+            PLOG(PL_DETAIL,"ManetGraphMLParser::WriteLocalAttributes():key=\"%s\",value=\"%s\"\n",attr->GetIndex(),attr->GetValue());
+            attr = it.GetNextItem();
+        }
+    }
+    return rv;
+}
+
+bool ManetGraphMLParser::WriteLocalInterfaceAttributes(xmlTextWriter* writerPtr,NetGraph::Interface& theInterface)
 {
     bool rv = true;
     char key[255];//this should be dynamic or checks added TBD
@@ -1473,7 +1700,9 @@ ManetGraphMLParser::WriteLocalInterfaceAttributes(xmlTextWriter* writerPtr,NetGr
         if(strcmp(attr->GetLookup(),key))
         {
             attr = NULL;
-        } else {
+        } 
+        else 
+        {
             rv += xmlTextWriterStartElement(writerPtr, BAD_CAST "data");
             rv += xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "key",BAD_CAST attr->GetIndex());
             rv += xmlTextWriterWriteString(writerPtr, BAD_CAST attr->GetValue());
@@ -1483,8 +1712,8 @@ ManetGraphMLParser::WriteLocalInterfaceAttributes(xmlTextWriter* writerPtr,NetGr
     }
     return rv;
 }
-bool 
-ManetGraphMLParser::WriteLocalLinkAttributes(xmlTextWriter* writerPtr,NetGraph::Link& theLink)
+
+bool ManetGraphMLParser::WriteLocalLinkAttributes(xmlTextWriter* writerPtr,NetGraph::Link& theLink)
 {
     PLOG(PL_DETAIL,"ManetGraphMLParser::WriteLocalLinkAttributes()\n");
     bool rv = true;
@@ -1502,7 +1731,9 @@ ManetGraphMLParser::WriteLocalLinkAttributes(xmlTextWriter* writerPtr,NetGraph::
         if(strcmp(attr->GetLookup(),key))
         {
             attr = NULL;
-        } else {
+        } 
+        else 
+        {
             rv += xmlTextWriterStartElement(writerPtr, BAD_CAST "data");
             rv += xmlTextWriterWriteAttribute(writerPtr, BAD_CAST "key",BAD_CAST attr->GetIndex());
             rv += xmlTextWriterWriteString(writerPtr, BAD_CAST attr->GetValue());
